
	;; =========================================================================================
	;; CLASE ABSTRACTA QUE DEFINE UN ACTOR DEL JUEGO
	;; =========================================================================================

	;; * Hereda de la estructura de datos de Sprite (ver actor_h.asm)

	;; * Incluye metodos para soporte de movimiento y colisiones

	;; * Los movimientos route son movimientos predefinidos en una lista cuyos datos son
	;;   los desplazamientos relativos (se suman a la coordenada actual). #80 indica el final.
	;;   Se pueden aplicar independientemnte en horizontal y vertical.

	;; CODIGO ========================================================

	;; Metodos principales:

	;; * actor_init				-> Inicializacion base de un actor
	;; * actor_calcScrX 			-> Calcula la coordenada X de pantalla del actor
	;; * actor_calcScrY			-> Calcula la coordenada Y de pantalla del actor
	;; * actor_process_h_motion		-> Procesa el movimiento de horizontal checkeando colisiones
	;; * actor_process_gravity_motion 	-> Procesa el movimiento de gravedad (solo para actors con gravedad)
	;; * actor_setCollisionHParams		-> Establece coordenadas en horizontal para colisiones (tanto de tiles como de sprites)
	;; * actor_setCollisionHParams_q 	-> Version de actor_setCollisionHParams con parametros simplifacdos
	;; * actor_setCollisionHParams_red 	-> Establece coordenadas en horizontal para colisiones (solo sprites)
	;; * actor_setCollisionHParams_red_q 	-> Version de actor_setCollisionHParams_red con parametros simplifacdos
	;; * actor_init_route_y 		-> Inicia un movimiento de recorrido especifico en Y
	;; * actor_process_route_y		-> Procesa un recorrido sobre la coordenada Y
	;; * actor_process_route_y_q		-> Version de actor_process_route_y con parametros simplifacdos

	;; ----------------------------------------------------------------

	;; INICIALIZACION ------------------------------------------------

	;; Inicializacion base de un actor

actor_init
	call	actor_calcScrX
	call	actor_calcScrY

	call	actor_setCollisionHParams
	jp	actor_setCollisionVParams

	;; Calcula la coordenada X de pantalla del actor

	;; Entrada:
	;; 	IX = actor info

	;; Salida:
	;;	HL = spr_x_scr_off

actor_calcScrX
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)
	ld	de,(h_scroll_pos)
	or	a
	sbc	hl,de
	ld	(ix + spr_x_scr_off),l
	ld	(ix + spr_x_scr_off + 1),h
	ret

	;; Calcula la coordenada Y de pantalla del actor

	;; Entrada:
	;; 	IX = actor info

actor_calcScrY
	ld	a,(ix + actor_y_world_h_off)
	ld	(ix + spr_y_scr_off),a
	ret

	;; MOVIMIENTO ----------------------------------------------------

	;; Procesa el movimiento de horizontal checkeando colisiones

	;; Entrada:
	;;	IX = puntero al actor
	;;	 A = pixeles de desplazamiento
	;; Salida:
	;;	Flag C = 1 si hay colision
	;;		A = tipo de colision
	;;		HL = puntero al ID del tile en que colision

actor_process_h_motion
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)

	or	a
	jr	z, actor_no_h_motion_required

	;; sin movimiento horizontal

;	bit	CAMERA_TRACKING_BIT,(ix + actor_flags_off)
;	ret	z				;; vuelve con Flag C = 0

;	call	refresh_scroll			;; solo refresca el scroll, (no hace falta llamar luego a actor_setCollisionHParams porque no se movio)

;	or	a				;; vuelve con Flag C = 0
;	ret					

;actor_h_motion_required

	;; con movimiento horizontal

	ld	e,a				;; E = valor de desplazamiento (+/-)
	EXT_____DE				;; extiende el signo de E en DE
	add	hl,de

	bit	7,d
	jr	nz, actor_move_left

actor_no_h_motion_required

	;; movimiento a la derecha

	push	hl				;; guarda actor_x_world_off
	call	actor_setCollisionHParams_q	;; establece los valores para la colision horizontal provisionalmente, ya que si hay colision puede deshacer el movimiento
	call	actor_checkTileCollision_right
	pop	hl				;; recupera actor_x_world_off

	or	a				;; Salida A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	jr	z, actor_move_h_no_collision

	;; colision por la derecha

	ld	bc,RIGHT_COLLISION * #100 + 1	;; + 1 para volver con Flag C = 1

	neg

	;; colision izq/der

actor_move_h_on_collision
	push	bc

	ld	e,a				;; E = valor de desplazamiento (+/-)
	EXT_____DE				;; extiende el signo de E en DE
	add	hl,de

	ld	(ix + actor_x_world_off),l
	ld	(ix + actor_x_world_off + 1),h

	bit	CAMERA_TRACKING_BIT,(ix + actor_flags_off)
	jr	z, actor_move_h_on_collision_no_refresh_scroll

	push	hl
	call	refresh_scroll_q
	pop	hl

actor_move_h_on_collision_no_refresh_scroll

	push	hl
	call	actor_setCollisionHParams_q
	pop	hl

	pop	af
	ret

	;; movimiento a la izquierda
	
actor_move_left
	push	hl				;; guarda actor_x_world_off
	call	actor_setCollisionHParams_q	;; establece los valores para la colision horizontal provisionalmente, ya que si hay colision puede deshacer el movimiento
	call	actor_checkTileCollision_left
	pop	hl				;; recupera actor_x_world_off

	ld	bc,LEFT_COLLISION * #100 + 1	;; + 1 para volver con Flag C = 1

	or	a				;; Salida A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	jr	nz, actor_move_h_on_collision

	; sin colision izq/der

actor_move_h_no_collision
	ld	(ix + actor_x_world_off),l
	ld	(ix + actor_x_world_off + 1),h

	bit	CAMERA_TRACKING_BIT,(ix + actor_flags_off)
	jr	z, actor_move_h_finish

	push	hl
	call	refresh_scroll_q		;; al llamar aqui, luego debe llamar a actor_setCollisionHParams_q
	pop	hl
	call	actor_setCollisionHParams_q

actor_move_h_finish
	or	a
	ret

	;; Procesa el movimiento de gravedad (solo para actors con gravedad)

	;; Entrada:
	;;	IX = puntero al actor
	;; Salida:
	;;	Flag C = 1 si hay colision
	;;	     A = tipo de colision
	;;	     HL = puntero al ID del tile en que colision

actor_process_gravity_motion
	ld	a,(ix + actor_gravity_state_off)
	ld	hl,actor_process_gravity_motion_per_state
	ADD_____HL_A
	jp	(hl)

actor_process_gravity_motion_per_state

	;; +0 -> ACTOR_GRAVITY_STATE_WEIGHTLESSNESS

	or	a					;; Flag C = 0 -> sin colision
	ret

	;; +2 -> ACTOR_GRAVITY_STATE_GOING_UP

	jr	actor_process_gravity_motion_going_up

	;; +4 -> ACTOR_GRAVITY_STATE_GOING_DOWN

	;; Procesa el movimiendo de caida del actor segun la gravedad

actor_process_gravity_motion_going_down
	call	actor_apply_gravity
	call	actor_setCollisionVParams
	call	actor_checkTileCollision_bottom

	or	a					;; Salida -> A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	jr	z, actor_on_gravity_motion_falling

	;; colision por abajo

	ex	af,af'

	ld	a,(ix + actor_state_off)
	cp	ACTOR_STATE_JUMPING
	jr	nz, $+6
	ld	(ix + actor_state_off),ACTOR_STATE_STOPPED	;; si estaba saltando (o cayendo) deja de estarlo

	ex	af,af'

	neg

	ld	bc,BOTTOM_COLLISION				;; OJO!, pone B a 0

actor_on_gravity_motion_collision
	ld	(ix + actor_vertical_impulse_off),b
	ld	(ix + actor_vertical_impulse_off + 1),b

	add	(ix + actor_y_world_h_off)
	ld	(ix + actor_y_world_h_off),a

	call	actor_setCollisionVParams_q

	ld	a,c
	scf

	;; HL = puntero al ID del tile en el que colision
	;; A = tipo de colision
	;; Flag C = 1

	ret

actor_on_gravity_motion_falling
	ld	(ix + actor_state_off),ACTOR_STATE_JUMPING
	; vuelve con Flag C = 0
	ret

	;; Procesa el movimiendo hacia arriba del actor por un salto

actor_process_gravity_motion_going_up
	call	actor_apply_gravity

	bit	7,h
	jr 	nz, $+6					;; cambio de direccion, ahora va hacia abajo
	ld	(ix + actor_gravity_state_off),ACTOR_GRAVITY_STATE_GOING_DOWN

	call	actor_setCollisionVParams
	call	actor_checkTileCollision_top
	or	a					;; Salida: A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	ret	z					;; vuelve con Flag C = 0

	;; colision por arriba

	ld	(ix + actor_gravity_state_off),ACTOR_GRAVITY_STATE_GOING_DOWN

	ld	bc,TOP_COLLISION			;; OJO!, pone B a 0
	jr	actor_on_gravity_motion_collision

	;; Aplica el movimiento de caida por gravedad

	;; Entrada:
	;;	IX = puntero al actor

	;; Salida:
	;;	HL = actor_vertical_impulse_off

actor_apply_gravity
	ld	l,(ix + actor_vertical_impulse_off)
	ld	h,(ix + actor_vertical_impulse_off + 1)
	ld	b,h
	ld	c,l

	ld	e,(ix + actor_y_world_off)		;; actor_y_world_off (8 bits de coma fija)
	ld	d,(ix + actor_y_world_h_off)		
	add	hl,de

	;; comprueba el tope por arriba

	ld	a,h
	ld	d,(ix + spr_pivot_y_off)		;; + pivotY
	add	d
	cp	GAMEAREA_H_PIXEL
	jr	c, actor_apply_gravity_continue
	add	(ix + spr_height_off)
	jp	m, actor_apply_gravity_continue
	xor	a
	sub	d
	ld	h,a
	ld	l,0	
actor_apply_gravity_continue

	ld	(ix + actor_y_world_off),l
	ld	(ix + actor_y_world_h_off),h

	ld	hl,GRAVITY_SPEED
	add	hl,bc
	bit	7,h
	jr	nz, actor_apply_gravity_uping

	ld	a,h
	cp	VERTICAL_IMPULSE_MAX / #100
	jr	c, $+5
	ld	hl,VERTICAL_IMPULSE_MAX

actor_apply_gravity_uping
	ld	(ix + actor_vertical_impulse_off),l
	ld	(ix + actor_vertical_impulse_off + 1),h

	ret

	;; Inicia un movimiento de recorrido especifico en Y

	;; Entrada:
	;; 	DE = puntero a la lista de desplazamientos relativos

actor_init_route_y
	set	Y_ROUTE_ACTIVE_BIT,(ix + actor_flags_off)
	ld	(ix + actor_route_y_off),e
	ld	(ix + actor_route_y_off + 1),d
	ret

	;; Procesa un recorrido sobre la coordenada Y

	;; Salida:
	;;	Flag C = 1 si termino el recorrido

actor_process_route_y
	or	a					;; flag C = 0
	bit	Y_ROUTE_ACTIVE_BIT,(ix + actor_flags_off)
	ret	z

actor_process_route_y_q

	;; avanza en recorrido precalculado en Y (saltos, caidas, ...)

	ld	e,(ix + actor_route_y_off)
	ld	d,(ix + actor_route_y_off + 1)
	ld	a,(de)
	cp	ROUTE_END_CODE				;; fin del recorrido?
	jr	z, actor_route_y_ends

	inc	de
	ld	(ix + actor_route_y_off),e
	ld	(ix + actor_route_y_off + 1),d

	add	(ix + actor_y_world_h_off)
	ld	(ix + actor_y_world_h_off),a
	or	a					;; flag C = 0
	ret

actor_route_y_ends
	res	Y_ROUTE_ACTIVE_BIT,(ix + actor_flags_off)
	scf						;; flag C = 1
	ret

	;; COLISIONES ----------------------------------------------------

	;; Establece coordenadas en horizontal para colisiones (tanto de tiles como de sprites)

	;; Entrada:
	;;	IX = puntero al actor

actor_setCollisionHParams
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)

	;; Entrada:
	;;	HL = actor_x_world_off
	;;	IX = puntero al actor

actor_setCollisionHParams_q
	ld	de,(h_scroll_pos)
	or	a
	sbc	hl,de

	ld	a,e
	and	%011
	ld	e,a

	ld	(ix + spr_x_scr_off),l			;; guarda x_scr por si no hay colision, ya lo tiene calculado
	ld	(ix + spr_x_scr_off + 1),h

	ld	a,l
	add	(ix + actor_collisionbb_xr_off)
	ld	(ix + actor_collisionbb_x1_off),a	;; para colision bounding box con sprites
	ld	d,a

	add	(ix + actor_collisionbb_w_off)
	ld	(ix + actor_collisionbb_x2_off),a	;; para colision bounding box con sprites

	add	e
	srl	a
	srl	a
	ld	(ix + actor_collisionbb_xt2_off),a
	ld	l,a

	ld	a,d
	add	e
	srl	a
	srl	a
	ld	(ix + actor_collisionbb_xt1_off),a

	sub	l
	neg
	ld	(ix + actor_collisionbb_wt_off),a

	ret

	;; Establece coordenadas en horizontal para colisiones (solo sprites)

	;; Entrada:
	;;	IX = puntero al actor
	;; Salida:
	;;	HL = spr_x_scr_off

actor_setCollisionHParams_red
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)

	;; Entrada:
	;;	HL = actor_x_world_off
	;;	IX = puntero al actor
	;; Salida:
	;;	HL = spr_x_scr_off

actor_setCollisionHParams_red_q
_h_scroll_pos
	ld	de,0 ;(h_scroll_pos)
	or	a
	sbc	hl,de

	ld	(ix + spr_x_scr_off),l			;; guarda x_scr por si no hay colision, ya lo tiene calculado
	ld	(ix + spr_x_scr_off + 1),h

	ld	a,l
	add	(ix + actor_collisionbb_xr_off)
	ld	(ix + actor_collisionbb_x1_off),a	;; para colision bounding box con sprites

	add	(ix + actor_collisionbb_w_off)
	ld	(ix + actor_collisionbb_x2_off),a	;; para colision bounding box con sprites

	ret

	;; Establece coordenadas en vertical para colisiones (tanto de tiles como de sprites)

	;; Entrada:
	;;	IX = puntero al actor

actor_setCollisionVParams
	ld	a,(ix + actor_y_world_h_off)

actor_setCollisionVParams_q
	ld	(ix + spr_y_scr_off),a			;; guarda y_scr por si no hay colision, ya lo tiene calculado

	add	(ix + actor_collisionbb_yr_off)
	ld	b,a
	ld	(ix + actor_collisionbb_y1_off),a	;; para colision bounding box con sprites
	add	(ix + actor_collisionbb_h_off)
	ld	(ix + actor_collisionbb_y2_off),a	;; para colision bounding box con sprites

	and	%11111000
	rra
	rra
	rra
	ld	(ix + actor_collisionbb_yt2_off),a

	srl	b
	srl	b
	srl	b
	ld	(ix + actor_collisionbb_yt1_off),b

	sub	b
	ld	(ix + actor_collisionbb_ht_off),a

	ret

	;; Establece coordenadas en vertical para colisiones (solo sprites)

	;; Entrada:
	;;	IX = puntero al actor

actor_setCollisionVParams_red
	ld	a,(ix + actor_y_world_h_off)

actor_setCollisionVParams_red_q
	ld	(ix + spr_y_scr_off),a			;; guarda y_scr por si no hay colision, ya lo tiene calculado

	add	(ix + actor_collisionbb_yr_off)
	ld	(ix + actor_collisionbb_y1_off),a	;; para colision bounding box con sprites
	add	(ix + actor_collisionbb_h_off)
	ld	(ix + actor_collisionbb_y2_off),a	;; para colision bounding box con sprites

	ret

	;; Comprueba si hay colision del actor con los tiles por arriba

	;; Entrada:
	;;	IX = actor

	;; Salida:
	;;	 A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	;;	HL = puntero al tile con el que colisiono

actor_checkTileCollision_top
	ld	a,(ix + actor_collisionbb_xt1_off)

	LD______DE_32A					;; DE  = A * 32

	ld	hl,(decompTileMap_adr)
	add	hl,de
	res	3,h
	ld	a,(ix + actor_collisionbb_yt1_off)
	add	l
	ld	l,a

	bit	7,(hl)
	jr	nz, actor_checkTileCollision_top_collision

	ld	a,(ix + actor_collisionbb_wt_off)
	or	a
	ret	z					;; sin colision?

	ld	de,DECOMP_TILE_MAP_H			;; siguiente columna en el buffer descomprimido del mapeado

actor_checkTileCollision_top_loop
	add	hl,de					;; siguiente columna
	res	3,h
	bit	7,(hl)
	jr	nz, actor_checkTileCollision_top_collision
	dec	a
	jr	nz,actor_checkTileCollision_top_loop

	ret						;; sin colision

actor_checkTileCollision_top_collision
	ld	a,(ix + actor_collisionbb_y1_off)
	and	7
	sub	8
	neg

	ret

	;; Comprueba si hay colision del actor con los tiles por abajo

	;; Entrada:
	;;	IX = actor

	;; Salida:
	;;	 A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	;;	HL = puntero al tile con el que colisiono

actor_checkTileCollision_bottom
	ld	a,(ix + actor_collisionbb_xt1_off)

	LD______DE_32A					;; DE = A * 32

	ld	hl,(decompTileMap_adr)
	add	hl,de
	res	3,h
	ld	a,(ix + actor_collisionbb_yt2_off)
	add	l
	ld	l,a

	bit	7,(hl)
	jr	nz, actor_checkTileCollision_bottom_collision

	ld	a,(ix + actor_collisionbb_wt_off)
	or	a
	ret	z					;; sin colision?

	ld	de,DECOMP_TILE_MAP_H			;; siguiente columna en el buffer descomprimido del mapeado

actor_checkTileCollision_bottom_loop
	add	hl,de					;; siguiente columna
	res	3,h
	bit	7,(hl)
	jr	nz, actor_checkTileCollision_bottom_collision
	dec	a
	jr	nz, actor_checkTileCollision_bottom_loop

	ret						;; sin colision

actor_checkTileCollision_bottom_collision
	ld	a,(ix + actor_collisionbb_y2_off)
	and	7
	inc	a
	ret

	;; Comprueba si hay colision del actor con los tiles por la izquierda

	;; Entrada:
	;;	IX = actor

	;; Salida:
	;;	 A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	;;	HL = puntero al tile con el que colisiono

actor_checkTileCollision_left
	ld	a,(ix + actor_collisionbb_xt1_off)

	LD______DE_32A					;; DE = A * 32

	ld	hl,(decompTileMap_adr)
	add	hl,de
	res	3,h
	ld	a,(ix + actor_collisionbb_yt1_off)
	add	l
	ld	l,a

	bit	7,(hl)
	jr	nz, actor_checkTileCollision_left_collision

	ld	a,(ix + actor_collisionbb_ht_off)
	or	a
	ret	z					;; sin colision?

actor_checkTileCollision_left_loop
	inc	l					;; siguiente fila
	bit	7,(hl)
	jr	nz, actor_checkTileCollision_left_collision
	dec	a
	jr	nz, actor_checkTileCollision_left_loop

	ret						;; sin colision

actor_checkTileCollision_left_collision
	ld	a,(_h_scroll_t_adjust + 1)
	add	(ix + actor_collisionbb_x1_off)

	and	3
	sub	4
	neg
	ret

	;; Comprueba si hay colision del actor con los tiles por la derecha

	;; Entrada:
	;;	IX = actor

	;; Salida:
	;;	 A = 0 si no hay colision, >0 si hay colision (indicando el numero de pixeles que ha penetrado en el tile que colisiono)
	;;	HL = puntero al tile con el que colisiono

actor_checkTileCollision_right
	ld	a,(ix + actor_collisionbb_xt2_off)

	LD______DE_32A					;; DE = A * 32

	ld	hl,(decompTileMap_adr)
	add	hl,de
	res	3,h
	ld	a,(ix + actor_collisionbb_yt1_off)
	add	l
	ld	l,a

	bit	7,(hl)
	jr	nz, actor_checkTileCollision_right_collision

	ld	a,(ix + actor_collisionbb_ht_off)
	or	a
	ret	z					;; sin colision?

actor_checkTileCollision_right_loop
	inc	l					;; siguiente fila
	bit	7,(hl)
	jr	nz, actor_checkTileCollision_right_collision
	dec	a
	jr	nz,actor_checkTileCollision_right_loop

	ret						;; sin colision

actor_checkTileCollision_right_collision
	ld	a,(_h_scroll_t_adjust + 1)
	add	(ix + actor_collisionbb_x2_off)

	and	3
	inc	a
	ret
