
	;; =========================================================================================
	;; SOPORTE DE SCROLL
	;; =========================================================================================

	;; * El scroll cambia el hsync (CRTC R3) para tener precision de byte y se combina con 
	;;   cambios en la paleta de colores para alcanzar precision de pixel en mode 0.

	;; * Al cambiar el hsync, hace que solo sea compatible con el monitor del CPC o CRT TVs.

	;; * Los cambios de paleta limita a usar solo 4 colores para el fondo x4 franjas verticales de
	;;   pantalla (cada franja con color de fondo + negro + 2 colores propios). Los sprites tambien
	;;   sufren limitaciones, quedando la distribucion de la paleta asi:

	;;   color  0 -> fondo (comun a todas las franjas del fondo y sprites)
	;;   color  1 -> negro (comun a todas las franjas del fondo y sprites)
	;;   color  2 -> tono oscuro (comun a fondo y sprites pero cambia en cada franja del fondo)
	;;   color  3 -> tono claro (comun a fondo y sprites pero cambia en cada franja del fondo)
	;;   color  4 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  5 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  6 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  7 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  8 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  9 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color 10 -> exclusivo del fondo (comun a todas las franjas del fondo)
	;;   color 11 -> exclusivo del fondo (comun a todas las franjas del fondo)
	;;   color 12 -> exclusivo sprites
	;;   color 13 -> exclusivo sprites
	;;   color 14 -> exclusivo sprites
	;;   color 15 -> exclusivo sprites

	;; CODIGO =========================================================

	;; Metodos principales:

	;; * initScroll 	-> Inicializa el scroll
	;; * processScroll 	-> Procesa el scroll
	;; * refresh_scroll 	-> Mueve el scroll en funcion de donde esta el sprite al que sigue la camara
	;; * refresh_scroll_q 	-> Variante de refresh_scroll con el parametro de scroll ya en HL
	;; * setHScrollPos	-> Establece la variable de posicion del scroll (h_scroll_pos) a partir de la coordenada de un actor
	;; * setHScrollPos_q	-> Establece la variable de posicion del scroll (variante de setHScrollPos con el valor ya en HL)

	;; ----------------------------------------------------------------

	;; Procesa el scroll
	;; (debe llamarse con las interrupciones apagadas, justo despues de sincronizar con el VB)

processScroll

	;; comprueba si se requiere de un nuevo scroll

_scrolled
	ld	a,0
	or	a
	jr	nz, setScrollValues

	;; sin scroll en este frame, solo conmuta el double buffer

	ld	hl,(h_scroll_pos)

	repeat	2

	sra	h
	rr	l

	rend

	jr	switchDoubleBuffer

	;; Inicializa el scroll

	;; Entrada:
	;; 	IX = datos del actor al que sigue la camara

initScroll
	call	setHScrollPos

	xor	a
	ld	(_scrolled + 1),a

	;; Establece los valores que producen el scroll

setScrollValues
	ld	de,(h_scroll_pos)

	ld	bc,setColors_bar0_l
	ld	hl,setColors_bar_l

	sra	d
	rr	e				;; bit 0
	jr	c, $+8
	ld	bc,setColors_bar0_r
	ld	hl,setColors_bar_r
	ld	(_bar0_hScroll + 1),bc		;; guarda los valores de la paleta
	ld	(_bar1_hScroll + 1),hl
	ld	(_bar2_hScroll + 1),hl
	ld	(_bar3_hScroll + 1),hl

	ld	a,R3_NO_SCROLL_VALUE
	sra	d
	rr	e				;; bit 1
	jr	c, $+4
	ld	a,R3_SCROLL_VALUE
	ld	(_reg3_value + 1),a		;; guarda el valor de R3

	ex	hl,de

	;; Conmuta el double buffer

switchDoubleBuffer

	ld	a,h
	and	%011
	ld	h,a				;; lo guarda en h

_double_buffer_value
	ld	a,GAMEAREA_SCREEN_R12		;; OJO, esto debe conmunar a %00100000 para double buffer
	xor	GAMEAREA_SCREEN_R12_SW
	ld	(_double_buffer_value + 1),a
	or	h

	ld	h,a
	ld	(_game_screen_crtc_adr + 1),hl	;; *** guarda los valores de R12 y R13

	;; otros cambios del double buffer

	ld	de,nDirtyTilesRect0
	ld	hl,dirtyTilesRect0_adr
	exx
	ld	de,nDirtyTilesRect1
	ld	hl,dirtyTilesRect1_adr

	ld	a,(_counterFrame + 1)
	and	1
	jr	nz,$+3
	exx

	ld	(_drawSprite_myDirtyTilesRect1 + 1),hl
	ld	(_drawSprite_myDirtyTilesRect2 + 1),hl
	ld	(_clearSprite_myDirtyTilesRect1 + 2),hl
	ld	(_clearSprite_myDirtyTilesRect2 + 2),hl
	ld	(_add_myDirtyTilesRect1 + 1),hl
	ld	(_add_myDirtyTilesRect2 + 1),hl

	ex	hl,de
	ld	(_drawSprite_nDirtyTilesRect + 1),hl
	ld	(_clearSprite_nDirtyTilesRect + 1),hl
	ld	(_add_nDirtyTilesRect + 1),hl

	exx

	ld	(_add_myDirtyTilesRect_repeat1 + 1),hl
	ld	(_add_myDirtyTilesRect_repeat2 + 1),hl
	ex	hl,de
	ld	(_add_nDirtyTilesRect_repeat + 1),hl

	ret

	;; Mueve el scroll en funcion de donde esta el sprite al que sigue la camara

	;; Entrada:
	;; 	IX = datos del actor que sigue la camara

refresh_scroll
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)

	;; Entrada:
	;; 	HL = actor_x_world_off del actor que sigue la camara

refresh_scroll_q
	ld	bc,-GAMEAREA_CENTER_X_PIXEL
	add	hl,bc
refresh_scroll_q2
	ld	de,(h_scroll_pos)
	ld	(h_scroll_pos),hl

	ld	b,h
	ld	c,l

	ld	a,e				;; guarda el valor anterior en A
	xor	l
	ld	(_scrolled + 1),a

	sbc	hl,de

	bit	2,a				;; bit 2 para precision de char
	jr	z, $+3				;; si no debe dibujar retorna con Flag NZ = 0

	ld	a,h				;; A = 0 -> columna de delante, <>0 -> columna de detras
						;; Flag NZ = 1
	ex	af,af'				;; *** GUARDA EL INDICADOR DE SI DEBE ACTUALIZAR UNA COLUMNA DEL SCROLL EN AF' PARA updateTileMap

	ld	a,c
	and	%011;%010
	ld	(_h_scroll_t_adjust + 1),a

	repeat	2

	sra	b
	rr	c

	rend

	ld	a,b
	and	%011
	ld	h,a				;; lo guarda en h

	ld	a,(_double_buffer_value + 1)
	xor	GAMEAREA_SCREEN_R12_SW

	add	a

	or	h
	ld	h,a
	ld	l,c
	add	hl,hl
	ld	(screen_adr),hl			;; base de memoria de video

	jp	updateTileMap

	;; Establece la posicion del scroll a partir de la coordenada de un actor

	;; Entrada:
	;;	IX = puntero a datos del actor

setHScrollPos
	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)

	;; Entrada:
	;;	HL = posicion horizontal 

setHScrollPos_q
	ld	bc,-GAMEAREA_CENTER_X_PIXEL ;-GAMEAREA_W_PIXEL 
	add	hl,bc
	ld	(h_scroll_pos),hl
	ret

