
	;; =========================================================================
	;; SPRITE
	;; =========================================================================

	;; * Soporta dibujado de sprites con o sin mascara

	;; * Incluye clipping por abajo. En horizontal y por arriba no se dibuja.

	;; * Al tener precision de pixel, es necesario tener 2 versiones de cada sprite,
	;;   una para dibujarse en pixel par y otra en impar.

	;; * Al dibujar un sprite, se guarda la info de los tiles de DECOMP_TILE_MAP que pisa
	;;   para el borrado

	;; * Incluye soporte para animacion de sprites. 
	;;   Las animaciones se definen con 2 variables ID: animacion y secuencia
	;;   Por animacion se entiende un conjunto de secuencias que corresponden a una accion,
	;;   por ejemplo: andando, volando, muriendo, etc...
	;;   Por secuencia se entiende un frame concreto de una animacion


	;; Metodos principales:

	;; * initSprites		-> Inicializa el soporte de sprites
	;; * drawSprite			-> Dibuja un sprite
	;; * sprite_process_anim	-> Procesa la animacino de un sprite
	;; * nextSpriteSec		-> Avanza la secuencia de animacion actual
	;; * setSpriteSec		-> Establece la secuencia dentro de la animacion actual
	;; * setSpriteAnimSec		-> Establece la animacion y secuencia del sprite
	;; * setSpriteAnim		-> Establece la animacion del sprite manteniendo la secuencia actual


	;; MACROS ========================================================

	;; IMPRESION DE 1 BYTE CON MASCARA -------------------------------

	MACRO	DRAW_MSK_BYTE			;; 10c

	;; B = tabla de mascara (byte alto) sprite_msk_table/#100
	;; DE = GFX
	;; HL = pantalla

	ld	a,(de)				;; 2c
	ld	c,a				;; 1c	; C = byte gfx
	ld	a,(bc)				;; 2c	; A = byte msk
	and	(hl)				;; 2c
	or	c				;; 1c
	ld	(hl),a				;; 2c

	MEND

	;; IMPRESION DE 1 BYTE CON MASCARA Y FLIP HORIZONTAL ----------------------------

;	MACRO	DRAW_MSK_HFLIP_BYTE		;; 14c

	;; H = tabla de mascara (byte alto) (justo despues esta la tabla de inversion)
	;; DE = GFX
	;; BC = pantalla

;	ld	a,(de)				;; 2c
;	ld	l,a				;; 1c	; L = byte gfx
;	inc	h				;; 1c
;	ld	l,(hl)				;; 2c	; L = byte gfx flippeado
;	dec	h				;; 1c
;	ld	a,(bc)				;; 2c	; A = byte de pantalla
;	and	(hl)				;; 2c	; and MSK
;	or	l				;; 1c	;  or GFX
;	ld	(bc),a				;; 2c

;	MEND

	;; AVANZA UN BYTE DE PANTALLA (HL) -----------------------------------------------

	MACRO	NEXT_BYTE_HL			;; 4/10/13c

	inc	l				;; 1c
	jr	nz, @_skip			;; 2/3c
	inc	h				;; 1c
	ld	a,h				;; 1c
	and	%0111				;; 2c
	jr	nz, @_skip			;; 2/3c
	ld 	a,h				;; 1c
	sub 	8				;; 2c
	ld	h,a				;; 1c
@_skip

	MEND

	;; AVANZA UN BYTE DE PANTALLA (DE) ------------------------------------------------

	MACRO	NEXT_BYTE_DE			;; 4/10/13c

	inc	e				;; 1c
	jr	nz, @_skip			;; 2/3c
	inc	d				;; 1c
	ld	a,d				;; 1c
	and	%0111				;; 2c
	jr	nz, @_skip			;; 2/3c
	ld 	a,d				;; 1c
	sub 	8				;; 2c
	ld	d,a				;; 1c
@_skip

	MEND

	;; SALTO DE SCANLINE PARA IMPRESION SIN MASCARA -----------------------------------------------

	MACRO	NEXT_SCANLINE_DE		;; 13/23c

	;; IYH = E al comienzo de la linea
	;; HL' = DE al comienzo de la linea
	;; DE' = NEXT_SCANLINE_MOD
	;; B' = 8
	;; C' = #38 (%0111000)

	ld	e,iyh				;; 2c		; 15/23c
	exx					;; 1c

	ld	a,h				;; 1c
	add	b				;; 1c		; B = 8
	ld	h,a				;; 1c
	and	c				;; 1c		; C = #38
	jr	nz, @_skip			;; 2/3c	

	add	hl,de				;; 3c
	res	3,h				;; 2c
	ld	a,l				;; 1c
	ld	iyh,a				;; 2c
	exx					;; 1c
	ld	e,a				;; 1c
	exx					;; 1c
@_skip
	ld	a,h				;; 1c
	exx					;; 1c
	ld	d,a				;; 1c

	MEND

	MACRO	NEXT_SCANLINE_HL		;; 14/24c

	;; IYH = L al comienzo de la linea
	;; HL' = HL al comienzo de la linea
	;; DE' = NEXT_SCANLINE_MOD
	;; B' = 8
	;; C' = #38 (%0111000)

	ld	a,iyh				;; 2c
	ld	l,a				;; 1c

	exx					;; 1c

	ld	a,h				;; 1c
	add	b				;; 1c		; B = 8
	ld	h,a				;; 1c
	and	c				;; 1c		; C = #38
	jr	nz, @_skip			;; 2/3c	

	add	hl,de				;; 3c
	res	3,h				;; 2c
	ld	a,l				;; 1c
	ld	iyh,a				;; 2c
	exx					;; 1c
	ld	l,a				;; 1c
	exx					;; 1c
@_skip
	ld	a,h				;; 1c
	exx					;; 1c
	ld	h,a				;; 1c

	MEND


	;; CODIGO ========================================================

	;; Inicializa el soporte de sprites

initSprites

	;; genera la tabla de mascaras

	ld	hl,sprite_msk_table
	ld	de,%1010101001010101
genMskTable_loop
	ld	c,0
	ld	a,d
	and	l
	jr	nz, $+3
	ld	c,d
	ld	a,e
	and	l
	ld	a,c
	jr	nz, $+3
	or	e
	ld	(hl),a
	inc	l
	jr	nz, genMskTable_loop

	;; genera la tabla de flipping

;	inc	h
;genFlippingTable_loop
;	ld	a,l
;	rrca
;	and	e
;	ld	c,a
;	ld	a,l
;	add	a
;	and	d
;	or	c
;	ld	(hl),a
;	inc	l
;	jr	nz, genFlippingTable_loop

	;; genera la tabla de coordenadas Y

	xor	a
	ld	e,a
	ld	d,#40
	ld	hl,sprite_addy_table
	ld	bc,GAMEAREA_H_PIXEL * #100 + 8
genAddYTable_loop
	ld	(hl),e
	inc	l
	ld	(hl),a
	inc	hl

	add	c
	cp	d
	jr	c, genAddYTable_continue

	ex	hl,de
	ld	h,a
	ld	a,b
	ld	bc,#c000 + GAMEAREA_W_BYTE
	add	hl,bc
	ld	b,a
	ld	a,h
	ex	hl,de
	ld	c,8
	ld	d,#40

genAddYTable_continue
	djnz	genAddYTable_loop

	ret

	;; Incluye las rutinas especificas segun los anchos de sprite soportados

	IF	WIDTH_1B_SUPPORT
	read	"sprite1B.asm"
	ENDIF

	IF	WIDTH_2B_SUPPORT
	read	"sprite2B.asm"
	ENDIF

	IF	WIDTH_4B_SUPPORT
	read	"sprite4B.asm"
	ENDIF

	IF	WIDTH_6B_SUPPORT
	read	"sprite6B.asm"
	ENDIF


drawSprite_hClipped
	scf
	ret

	;; Dibuja un sprite

	;; Entrada:
	;; 	IX = sprite info

	;; Salida:
	;;	Flag C = 1 si no se dibujo por estar fuera de los margenes laterales

drawSprite

	;; pivotY

	ld	a,(ix + spr_y_scr_off)
	add	(ix + spr_pivot_y_off)			;; + pivotY
	ld	b,a					;; B = Y (parte entera)

	;; clipping vertical

	sub	GAMEAREA_H_PIXEL
	ret	nc

	ld	d,(ix + spr_height_off)			;; H = altura

	neg
	cp	d
	jr	nc, $+3 				;; sin clipping
	ld	d,a					;; clipping bottom

	ld	iyl,d					;; *** IYL = altura

	;; pivotX

	ld	h,(ix + spr_x_scr_off + 1)
	ld	l,(ix + spr_x_scr_off)

	ld	e,(ix + spr_pivot_x_off)
	EXT_____DE
	add	hl,de

	;; clipping left

	bit	7,h
;	ret	nz
	jr	nz, drawSprite_hClipped

	ld	c,l

	;; clipping right

	ld	a,(ix + spr_tile_width_off)
	ld	d,a
	add	a
	add	a
	sub	GAMEAREA_W_PIXEL
	ld	e,a
	ld	a,d
	ld	d,#ff
	add	hl,de
	
	ret	c
	ld	d,a

	;; guardar la siguiente informacion para el borrado:

	;; ( + 0) .w = posicion del tile de la esquina superior izquierda en DECOMP_TILE_MAP
	;; ( + 2) .b = w
	;; ( + 3) .b = h
	;; ( + 4) .w = posicion en pantalla

_drawSprite_myDirtyTilesRect1
	ld	hl,(0)
	dec	hl
	dec	l
	ld	(_drawSprite_save_screen + 2),hl	;; ( + 4) .w = posicion en pantalla

	ld	a,b					;; A = Y
	and	%0111
	ld	e,a

	ld	a,iyl
	dec	a
	and	%0111
	add	e
	cp	8
	ld	a,iyl
	jr	c,$+4
	add	8
	add	7

	and	%11111000
	rra						;; /2
	rra						;; /4
	rra						;; /8

	dec	hl
	ld	(hl),a					;; ( + 3) .b = H (en tiles)

	ex	af,af'					;; *** guarda H (en tiles) en A'

_h_scroll_t_adjust
	ld	a,0
	and	%10					;; primer bit a 0 (precision de pixel) porque tanto piexels pares como impares se escriben en el mismo byte
	ld	iyh,a					;; guarda A en IYH

	add	c
;	or	c
	bit	1,a
	
;	ld	d,(ix + spr_tile_width_off)
	jr	z,$+3
	inc	d

	dec	l
	ld	(hl),d					;; ( + 2) .b = W (en tiles)

	dec	hl
	dec	l
	ld	(_drawSprite_save_tileMapPos + 1),hl
_drawSprite_myDirtyTilesRect2
	ld	(0),hl

;	ld	a,iyh
;	add	c					;; A = X
	and	%11111100
	ld	h,0
	ld	l,a
	add	hl,hl					;; x8
	add	hl,hl					;; x16
	add	hl,hl					;; x32

	ld	a,b					;; A = Y
	and	%11111000
	rra						;; /2
	rra						;; /4
	rra						;; /8
	add	l
	ld	l,a

	ld	a,b
	ex	af,af'					;; A = H (en tiles) y guarda B en A'
	ld	b,d					;; B = W (en tiles)

	ld	de,(decompTileMap_adr)
	add	hl,de
	res	3,h

_drawSprite_save_tileMapPos
	ld	(0),hl					;; ( + 0) .w = posicion del tile de la esquina superior izquierda en DECOMP_TILE_MAP

	;; aqui no se puede pisar BC, IY

	ld	e,(ix + spr_draw_mode_off)
	ld	d,drawSprite_mode_opaque / #100
	ex	hl,de
	jp	(hl)

	ALIGN	#10					;; se asegura que drawSprite_mode_opaque, drawSprite_mode_msk y drawSprite_mode_auto esten en el mismo bloque de #100 bytes en memoria

	;; modo opaco

drawSprite_mode_opaque
	ld	a,OPAQUE_SPRITE_OFF + sprite_routines AND #ff
	jr	drawSprite_mode_ok

	;; modo mascara

drawSprite_mode_msk
	ld	a,MASKED_SPRITE_OFF + sprite_routines AND #ff
	jr	drawSprite_mode_ok

	;; modo automatico

drawSprite_mode_auto
	ex	hl,de
	add	a
	neg
	add	drawSprite_chkBg_loop_end - _drawSprite_chkbg_loop_jr1 - 2
	ld	(_drawSprite_chkbg_loop_jr1 + 1),a
	ld	(_drawSprite_chkbg_loop_jr2 + 1),a

	ld	de,DECOMP_TILE_MAP_H			;; siguiente columna en el buffer descomprimido del mapeado
	xor	a
	jr	_drawSprite_chkbg_loop_jr1

drawSprite_chkbg_loop
	add	hl,de
	res	3,h
_drawSprite_chkbg_loop_jr1
	jr	$+2

	repeat	MAX_SPRITE_H + 7 / 8
	or	(hl)
	inc	l
	rend
	or	(hl)
	
	dec	b
drawSprite_chkbg_loop_end
	jr	z, drawSprite_chkbg_end

	add	hl,de
	res	3,h
_drawSprite_chkbg_loop_jr2
	jr	$+2

	repeat	MAX_SPRITE_H + 7 / 8

	or	(hl)
	dec	l

	rend

	or	(hl)

	djnz	drawSprite_chkbg_loop
drawSprite_chkbg_end

	or	a
	ld	a,OPAQUE_SPRITE_OFF + sprite_routines AND #ff
	jr	z, drawSprite_mode_ok
	ld	a,MASKED_SPRITE_OFF + sprite_routines AND #ff

drawSprite_mode_ok
	ld	(_drawSprite_mode + 1),a

	ex	af,af'					;; recupera B en A
	ld	d,a					;; guarda Y en D

;	ld	d,b					;; guarda Y en D

	;; incrementa el numero de rectangulos de tile sucios

_drawSprite_nDirtyTilesRect
	ld	hl,0
	inc	(hl)

	ld	l,(ix + spr_current_frame_off)		;; coge el frame de la cache
	ld	h,(ix + spr_current_frame_off + 1)

	;; selecciona frame par/impar dependiendo de la X del sprite

	srl	c
	jr	nc, $+4
	inc	l
	inc	l

	ld	a,iyh					;; recupera (h_scroll_pos) en A
	srl	a

	add	c
	ld	c,a
	ld	b,0					;; BC = addx

	ld	a,(ix + spr_width_off)			;; A = IY offset de rutinas segun el ancho
	bit	0,c
	jr	z, $+4
	add	SPRITE_BYTE_OFF				;; IY offset += 8

_drawSprite_mode
	add	MASKED_SPRITE_OFF + sprite_routines AND #ff
;	add	OPAQUE_SPRITE_OFF + sprite_routines AND #ff

	ld	e,a
;	ld	a,(ix + spr_y_scr_off)			;; Y (parte entera)
	ld	a,d
	ld	d,sprite_routines / #100

	push	de					;; *** guarda la rutina para llarmarla con RET

	ld	e,(hl)
	inc	l
	ld	d,(hl)					;; *** DE = GFX

	add	a					;; x2
	ld	l,a
	ld	a,sprite_addy_table / #100
	adc	b					;; OJO!, aqui B debe ser 0
	ld	h,a

	ld	a,(hl)
	inc	l
	ld	h,(hl)
	ld	l,a					;; HL = addy

	push	hl

_screen_adr
	ld	hl,0;(screen_adr)
	ld	a,h
	add	hl,bc					;; + addx

	pop	bc					;; recupera addy

	ex	af,af'
	ld	a,b
	ld	iyh,a
	and	%0111
	ld	b,a
	add	hl,bc					;; + addy
	ld	a,%11111000
	and	iyh
	ld	b,a
	ex	af,af'

	ld	c,a
	
	xor	h
	bit 	3,a
	ld 	a,h
	jr	z, $+4
	sub 	8

	add	b
	ld	h,a
	xor	c
	bit	6,a
	jr	z, $+6
	ld	bc,NEXT_SCANLINE_MOD
	add	hl,bc					;; *** HL = screen

;_drawSprite_save_screen
;	ld	(0),hl

	ld	a,h
	exx
	ld	h,a
	exx

	;; En los datos del sprite, falta un dato de 16 bits que sirve para detectar si el sprite se va a dibujar en zona de pantalla 
	;; no consecutiva y que, por tanto, va a necesitar incrementar los bytes de forma compleja (en vez de con un simple inc hl).
	;; Este dato es el alto maximo del sprite en caracteres (redondeado por arriba) * modulo de pantalla + ancho del sprite en bytes - 1.
	;; Ese valor se suma a la direccio de pantalla y si el bit 3 del byte mas significativo (o bit 11) es diferente tras la suma,
	;; es que necesita incrementar los bytes de forma compleja, sino lo podra hacer con "inc hl".

;;	ld	a,h
;;	ld	c,(ix+spr_lastByte_off)
;;	ld	b,(ix+spr_lastByte_off+1)
;;	push	hl
;;	add	hl,bc
;;	xor	h
;;	pop	hl
;;	bit	3,a
;;	jr 	z, drawSprite_simple_inc		; incremento simple
	;; incremento complejo
;;	pop	bc
;;	set	4,c
;;	push	bc
;;drawSprite_simple_inc


	ld	a,l
	exx
	ld	l,a					;; *** HL' = screen
	exx

	ld	iyh,a					;; *** IYH = screen (byte bajo)

	and	%11111110
	ld	c,a
	ld	a,h
	and	%11000111
	ld	b,a					;; DE' = posicion en pantalla (a nivel de tile)
_drawSprite_save_screen
	ld	(0),bc

	ld	b,sprite_msk_table / #100		;; *** B = tabla de mascara (byte alto) sprite_msk_table/#100

	;; B = tabla de mascara (byte alto)
	;; DE = GFX
	;; HL = pantalla
	;; HL' = pantalla
	;; IYH = pantalla (byte bajo)
	;; IYL = altura

	;; 	DE' = NEXT_SCANLINE_MOD	(viene de antes)
	;; 	B' = 8			(viene de antes)
	;; 	C' = #38 (%0111000)	(viene de antes)

	ret						; 3c

	LIST

	ALIGN	SPRITE_ROUTINES_ALIGN

sprite_routines

	NOLIST

	;; 1 BYTE DE ANCHO =====================================

	IF	WIDTH_1B_SUPPORT

	;; CON MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite1B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite1B_byte_odd

	;; SIN MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite1B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite1B_byte_odd

	ALIGN	SPRITE_SIZE_OFF

	ENDIF

	;; 2 BYTES DE ANCHO =====================================

	IF	WIDTH_2B_SUPPORT

	;; CON MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite2B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite2B_byte_odd

	;; SIN MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite2B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite2B_byte_odd

	ALIGN	SPRITE_SIZE_OFF

	ENDIF

	;; 4 BYTES DE ANCHO =====================================

	IF	WIDTH_4B_SUPPORT

	;; CON MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite4B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite4B_byte_odd

	;; SIN MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite4B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite4B_byte_odd

	ALIGN	SPRITE_SIZE_OFF

	ENDIF

	;; 6 BYTES DE ANCHO =====================================

	IF	WIDTH_6B_SUPPORT

	;; CON MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite6B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	jp	drawMskSprite6B_byte_odd

	;; SIN MASCARA ----------------------

	;; en byte par

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite6B_byte_even

	;; en byte impar

	ALIGN	SPRITE_BYTE_OFF
	ex	hl,de
	jp	drawSprite6B_byte_odd

	ENDIF



	;; SOPORTE DE ANIMACION DE SPRITES -----------------------------

	;; Procesa la animacino de un sprite

	;; Entrada:
	;; 	IX = sprite info
	;;  	B = AND que define la velocidad (0 = 50 fps, 1 = 25 fps, 3 = 12,5 fps, 7 = 6,25)

sprite_process_anim
	ld	a,(_counterFrame + 1)
	and	b;%011
	ret	nz

	;; Avanza la secuencia de animacion actual

	;; Entrada:
	;; 	IX = sprite info

nextSpriteSec
	ld	a,(ix + spr_sec_off)			;; A = sec
	inc	a

	ld	l,(ix + spr_current_anim_off)
	ld	h,(ix + spr_current_anim_off + 1)
	cp	(hl)
	inc	hl
	jr	c, $+3
	ld	a,(hl)

	ld	(ix + spr_sec_off),a
	jr	cacheSpriteSec2

	;; Establece la secuencia dentro de la animacion actual

	;; Entrada:
	;; 	IX = sprite info
	;;  	A = sec

setSpriteSec
	ld	(ix + spr_sec_off),a

	ld	l,(ix + spr_current_anim_off)
	ld	h,(ix + spr_current_anim_off + 1)

	jr	cacheSpriteSec


	;; Establece la animacion y secuencia del sprite

	;; Entrada:
	;; 	IX = sprite info
	;;  	A = anim
	;;  	B = sec

setSpriteAnimSec
	ld	(ix + spr_sec_off),b

	;; Establece la animacion del sprite manteniendo la secuencia actual

	;; Entrada
	;; 	IX = sprite info
	;;  	 A = anim
	;; Salida
	;;	HL = (ix + spr_current_frame_off)

setSpriteAnim
;	ld	(ix + spr_current_anim_off),a
	ld	(ix + spr_anim_off),a

	;; cachea el puntero a la info de la animacion para optimizar futuros accesos

	ld	l,(ix + spr_anims_off)
	ld	h,(ix + spr_anims_off + 1)		;; HL = animaciones
	ADD_____HL_A

	ld	e,(hl)
	inc	l
	ld	d,(hl)
	ex	hl,de

	ld	(ix + spr_current_anim_off),l
	ld	(ix + spr_current_anim_off + 1),h

	ld	a,(ix + spr_sec_off)			;; A = sec
cacheSpriteSec
	inc	a					;; se salta el numero de frames
cacheSpriteSec2
	inc	a					;; se salta el frame de inicio del loop

	ADD_____HL_A
	ld	a,(hl)					;; A = frame Id

	ld	l,(ix + spr_frames_off)
	ld	h,(ix + spr_frames_off + 1)
	ADD_____HL_A

	ld	(ix + spr_current_frame_off),l
	ld	(ix + spr_current_frame_off + 1),h

	ret


	;; Mueve un sprite con OPQA

	;; IX = puntero a datos del sprite

;moveSprite
;	ld	b,(ix + spr_x_scr_off)
;	ld	c,(ix + spr_y_scr_off)

;	CHECK_O_KEY
;	jr	nz, $+3
;	dec	b

;	CHECK_P_KEY
;	jr	nz, $+3
;	inc	b

;	CHECK_Q_KEY
;	jr	nz, $+3
;	dec	c

;	CHECK_A_KEY
;	jr	nz, $+3
;	inc	c

;	ld	(ix + spr_x_scr_off),b
;	ld	(ix + spr_y_scr_off),c

;	ret

