
	;; =========================================================================================
	;; ENEMIGO PAJARO (sin colision con el mapeado)
	;; =========================================================================================

	;; *  Si se encuentra fuera del area visible pero dentro del margen definido en BIRD_HMOV_AMP,
	;;    sigue activo y movimiendose.

	;; *  Si se encuentra fuera del area visible y a mas distancia que BIRD_HMOV_AMP, se
	;;    elimina.

	; CODIGO --------------------------------------------------------

	;; Metodos principales:

	;; * bird_initSupport		-> Inicializacion de soporte para enemigos tipo pajaros
	;; * bird_hi_create		-> Creacion de un enemigo tipo pajaro que vuela alto
	;; * bird_low_create		-> Creacion de un enemigo tipo pajaro que vuela un poco mas bajo
	;; * bird_process		-> Process de un pajaro


	;; Inicializacion de soporte para enemigos tipo pajaros

bird_initSupport

	;; Inicializa un buffer de actors

	ld	ix,myBirdsInfo			;; IX = puntero a lista de actors
	ld	e,BIRD_INFO_LEN			;;  E = tamanyo de la lista
	ld	b,N_BIRDS_MAX			;;  B = numero de actors maximo
	ld	hl,bird_fixed_info		;; HL = datos fijos
	jp	initActors

	;; Creacion de un enemigo tipo pajaro

	;; Entrada:
	;;	DE = puntero a guardar en enemy_gTileMap_adr_off
	;;	BC = coordenada X relativa donde debe poner el enemigo (0 / GAMEAREA_W_PIXEL)
	;; 	BC' = posicion en la pantalla de tiles (linea que va a dibujar)
	;; 	HL' = posicion en la pantalla de tiles (linea de la columna inversa con morralla)

	;; Variante que vuelta alto

bird_hi_create
	ld	hl,20*#100
	jr	bird_create

	;; Variante que vuela bajo

bird_low_create
	ld	hl,52*#100

bird_create
	ld	ix,myBirdsInfo
	call	enemy_create

	ld	(ix + actor_state_off),ACTOR_STATE_FLYING

	call	actor_init

	ld	de,small_sin_route		;; DE = puntero a la lista de desplazamientos relativos
	call	actor_init_route_y	

	ld	(ix + actor_speed_off),-1
	ld	(ix + bird_changeDir_counter_off),BIRD_HMOV_AMP

	ld	a,BIRD_FLY_LEFT_ANIM_ID		;;  A = anim
	ld	b,0				;;  B = sec
	jp	setSpriteAnimSec


	;; Process de un pajaro

	;; Entrada:
	;; 	IX = estructura de datos del pajaro

bird_process

	;; movimiento en horizontal

	ld	l,(ix + actor_x_world_off)
	ld	h,(ix + actor_x_world_off + 1)
	ld	e,(ix + actor_speed_off)
	EXT_____DE
	add	hl,de
	ld	(ix + actor_x_world_off),l
	ld	(ix + actor_x_world_off + 1),h

	call	actor_setCollisionHParams_red_q		;; actualiza la info para colision

	;; comprueba si se salio de la pantalla por los lados para eliminarlo

	ld	de,BIRD_HMOV_AMP
	add	hl,de
	bit	7,h
	jp	nz, enemy_remove

	ld	de,-GAMEAREA_W_PIXEL - BIRD_HMOV_AMP - BIRD_HMOV_AMP
	add	hl,de
	jp	c, enemy_remove

	;; movimiento en vertical

	call	actor_process_route_y_q
	jr	nc, bird_process_no_loop_route_y
	ld	de,small_sin_route
	call	actor_init_route_y
bird_process_no_loop_route_y
	call	actor_setCollisionVParams_red		;; actualiza la info para colision

	;; comprobacion de colisiones con el jugador

	call	checkPlayerActorCollision
	jp	c, enemy_remove

	;; comprueba si debe cambiar de direccion (ping-pong)

	dec	(ix + bird_changeDir_counter_off)
	jr	nz, bird_process_same_dir

	;; cambio de direccion ping-pong

	ld	(ix + bird_changeDir_counter_off),BIRD_HMOV_AMP
	xor	a
	sub	(ix + actor_speed_off)
	ld	(ix + actor_speed_off),a

	ld	a,BIRD_FLY_LEFT_ANIM_ID			;;  A = anim
	jp	m, $+5
	ld	a,BIRD_FLY_RIGHT_ANIM_ID
	call	setSpriteAnim
	jr	bird_process_dir_changed
bird_process_same_dir
	ld	b,3
	call	sprite_process_anim

bird_process_dir_changed

	;; dibuja el sprite

	call	drawSprite

	jp	actor_process_back
	

	;; DATOS ---------------------------------------------------------

			;; DATOS FIJOS DE LOS PAJAROS

bird_fixed_info		;; de sprite
			db	SPRITE_DRAW_MODE_AUTO		;;     = draw mode
			dw	bird_frames			;; +2  = frames
			dw	bird_anims			;; +4  = animaciones

			db	SPRITE_24W			;; +10  = ancho	(0 = 16, #10 = 20, ...)
			db	16				;; +11  = alto en pixeles
			db	24/8				;; +12 = ancho en tiles
			db	-5				;; +13 = pivotX
			db	-8				;; +14 = pivotY
			;; de actor
			db	-4				;; +34 = actor_collisionbb_xr_off
			db	-5				;; +35 = actor_collisionbb_yr_off
			db	8-2				;; +36 = actor_collisionbb_w_off
			db	10				;; +37 = actor_collisionbb_h_off

			;; ANIMACIONES

BIRD_FLY_RIGHT_ANIM_ID	EQU	0
BIRD_FLY_LEFT_ANIM_ID	EQU	2

			ALIGN 	2

bird_anims		dw	bird_fly_right			;; anim id 0 -> volando derecha
			dw	bird_fly_left			;; anim 1d 2 -> volando izquierda

			; ...

bird_fly_right		db	4				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0,   8,  16,  8			;; frame id (multiplo de 4)

bird_fly_left		db	4				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	4,  12,  20, 12			;; frame id (multiplo de 4)

			; ...

			ALIGN 	4

			;; DESCRIPCION DE FRAMES DE ANIMACION

bird_frames		;;	frame par			frame impar
			dw	spr_bird_fly01_par,		spr_bird_fly01_impar		;; frame id 0
			dw	spr_bird_fly01_i_par,		spr_bird_fly01_i_impar		;; frame id 4

			dw	spr_bird_fly02_par,		spr_bird_fly02_impar		;; frame id 8
			dw	spr_bird_fly02_i_par,		spr_bird_fly02_i_impar		;; frame id 12

			dw	spr_bird_fly03_par,		spr_bird_fly03_impar		;; frame id 16
			dw	spr_bird_fly03_i_par,		spr_bird_fly03_i_impar		;; frame id 20


