
	;; =========================================================================================
	;; MONEDA 
	;; =========================================================================================

	;; * Aunque usa la estructura de actor, no se dibuja como sprites sino con tiles via rectangulos
	;;   de tiles sucios, ya que asi no deben borrarse ni dibujarse con mascara y es mas rapido.

	;; * Para ahorrar memoria (y como pueden haber muchas a la vez en pantalla), algunos datos
	;;   de la estructura de actor que no se utilizan se sobrescriben con datos especificos de
	;;   las monedas.


	;; CODIGO --------------------------------------------------------

	;; Metodos principales:

	;; * coin_initSupport		-> Inicializacion de soporte para monedas
	;; * coin_row00_create		-> Crea una moneda en la fila 0
	;; * coin_row02_create		-> Crea una moneda en la fila 2
	;; * coin_row04_create		-> Crea una moneda en la fila 4
	;; * coin_row06_create		-> Crea una moneda en la fila 6
	;; * coin_row08_create		-> Crea una moneda en la fila 8
	;; * coin_row10_create		-> Crea una moneda en la fila 10
	;; * coin_row12_create		-> Crea una moneda en la fila 12
	;; * coin_row14_create		-> Crea una moneda en la fila 14
	;; * coin_processAll		-> Process de todas las monedas
	;; * coin_process		-> Process de una moneda


	; Inicializacion de soporte para monedas

coin_initSupport

	;; Inicializa un buffer de actors

	ld	ix,myCoinsInfo			;; IX = puntero a lista de actors
	ld	e,COIN_INFO_LEN			;;  E = tamanyo de la lista
	ld	b,N_COINS_MAX			;;  B = numero de actors maximo
	ld	hl,coin_fixed_info		;; HL = datos fijos
	jp	initActors

	;; Creacion de un enemigo tipo moneda

	;; Entrada:
	;;	DE = puntero a guardar en enemy_gTileMap_adr_off
	;;	BC = coordenada X relativa donde debe poner el enemigo (0 / GAMEAREA_W_PIXEL)
	;; 	BC' = posicion en la pantalla de tiles (linea que va a dibujar)
	;; 	HL' = posicion en la pantalla de tiles (linea de la columna inversa con morralla)
	;;	(SP + 2) = posicion en pantalla (arriba)

coin_row00_create
	xor	a				;; posicion Y de caracter
	ld	h,a				;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	l,a
	jr	coin_create

coin_row02_create
	ld	hl,2*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,2				;; posicion Y de caracter
	jr	coin_create

coin_row04_create
	ld	hl,4*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,4				;; posicion Y de caracter
	jr	coin_create

coin_row06_create
	ld	hl,6*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,6				;; posicion Y de caracter
	jr	coin_create

coin_row08_create
	ld	hl,8*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,8				;; posicion Y de caracter
	jr	coin_create

coin_row10_create
	ld	hl,10*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,10				;; posicion Y de caracter
	jr	coin_create

coin_row12_create
	ld	hl,12*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,12				;; posicion Y de caracter
	jr	coin_create

coin_row14_create
	ld	hl,14*GAMEAREA_W_BYTE		;; offset de pantalla relativo correspondiente a la posicion Y del caracter
	ld	a,14				;; posicion Y de caracter

coin_create
	push	hl					;; guarda el offset de pantalla relativo correspondiente a la posicion Y del caracter

	ld	l,a
	ex	af,af'
	ld	a,l

	add	a
	add	a
	add	a
	ld	h,a
	ld	l,0

	ld	ix,myCoinsInfo
	call	enemy_create				;; modifica A, BC, IX

	exx
	ex	af,af'
	add	c
	ld	(ix + coin_decompTileMap_adr_off),a
	ld	(ix + coin_decompTileMap_adr_off + 1),b ;; BA = posicion del tile de la esquina superior izquierda en DECOMP_TILE_MAP
	exx

	ld	(ix + actor_state_off),ACTOR_STATE_ROTATING

	;; establece las coordenadas del rectangulo de tiles

	pop	de					;; recupera el offset de pantalla relativo correspondiente a la posicion Y del caracter
	pop	bc					;; *** IMPORTANTE, conserva el valor de la pila (que es el retorno de esta subrutina) en BC, porque una interrupcion podria alterar la pila
	pop	hl					;; HL = posicion en pantalla (arriba)
	push	hl
	push	bc
	add	hl,de
	res	3,h

	ld	(ix + coin_scr_adr_off),l		
	ld	(ix + coin_scr_adr_off + 1),h		;; HL = posicion en pantalla

_coin_anim_mod
	ld	a,0					;; va distribuyendo el frame en que se anima cada moneda
	ld	(ix + coin_anim_mod_off),a
;	inc	a
	add	2
	and	3
	ld	(_coin_anim_mod + 1),a

	call	actor_init

	ld	a,COIN_ROTATION_ANIM_ID			;;  A = anim
	ld	b,0					;;  B = sec
	jp	setSpriteAnimSec


	;; Process de todas las monedas (optimiza cosas de coin_process)

coin_processAll

	;; optimizaciones previas en coin_process

	ld	a,(_counterFrame + 1)
	and	3
	ld	(_coin_frame_mod4 + 1),a

	ld	a,(screen_adr + 1)			;; la posicion en pantalla cambia segun el buffer de pantalla actual
	bit	6,a
	ld	a,#f0					;; set 6,b
	jr	nz, $+4
	ld	a,#b0					;; res 6,b
	ld	(_coin_draw_scrBuffer + 1),a

	ld	ix,myCoinsInfo				;; IX = puntero a lista de actors
	ld	hl,coin_process				;; HL = puntero a la rutina process del actor
	jp	processActorMng


	;; Process de una moneda

	;; Entrada:
	;; 	IX = estructura de datos de la moneda

coin_process
	call	actor_setCollisionHParams_red		;; devuelve HL = spr_x_scr_off

	;; comprobacion de colisiones con el jugador

	call	checkPlayerActorCollision
	jr	c, coin_taken

	;; comprueba si se salio de la pantalla por los lados para eliminarlo

	ld	de,COIN_LEFT1_MOV_MARGIN
	add	hl,de
	bit	7,h
	jr	nz, coin_remove				;; elimina la moneda por salirse de la pantalla

	ld	de,-COIN_LEFT1_MOV_MARGIN - COIN_LEFT2_MOV_MARGIN
	add	hl,de
	bit	7,h
	jr	nz, coin_process_back			;; deja de animarla para no sobrescribir en las columnas de tiles que aparcen con el scroll

;	ld	de,-GAMEAREA_W_PIXEL - COIN_RIGHT1_MOV_MARGIN + COIN_LEFT2_MOV_MARGIN
	ld	e,-GAMEAREA_W_PIXEL - COIN_RIGHT1_MOV_MARGIN + COIN_LEFT2_MOV_MARGIN
	add	hl,de
	jr	c, coin_remove;enemy_remove		;; elimina la moneda por salirse de la pantalla

	ld	de,COIN_RIGHT2_MOV_MARGIN
	add	hl,de
	jr	c, coin_process_back			;; deja de animarla para no sobrescribir en las columnas de tiles que aparcen con el scroll

	;; anima la moneda

_coin_frame_mod4
	ld	a,0
	cp	(ix + coin_anim_mod_off)
	jp	nz, actor_process_back

	;; avanza la animacion

	call	nextSpriteSec				;; devuelve HL = (ix + spr_current_frame_off)

	ld	a,(hl)
	inc	l
	ld	h,(hl)
	ld	l,a

	call	coin_draw

coin_process_back
	jp	actor_process_back

	;; el jugador coge la moneda

coin_taken
	ld	hl,coin_frame_blank
	call	coin_draw

	call	player_on_coin_collision
coin_remove
	jp	enemy_remove


	;; Dibuja los tiles de una moneda

	;; Entrada:
	;; 	HL = frame de tiles de la moneda

coin_draw

	;; pokea en DECOMP_TILE_MAP los tiles del siguiente frame de animacion

	ld	e,(ix + coin_decompTileMap_adr_off)
	ld	d,(ix + coin_decompTileMap_adr_off + 1) 	;; DE = posicion del tile de la esquina superior izquierda en DECOMP_TILE_MAP
	push	de

	repeat	COIN_N_TILES_H
	ldi
	rend

	repeat	COIN_N_TILES_W - 1

	ld	a,DECOMP_TILE_MAP_H - COIN_N_TILES_H
	add	e
	ld	e,a
	jr	nc, $+5
	inc	d
	res	3,d

	repeat	COIN_N_TILES_H
	ldi
	rend

	rend

	;; anyade un rectangulo de tiles sucios

	ld	c,(ix + coin_scr_adr_off)
	ld	b,(ix + coin_scr_adr_off + 1)
_coin_draw_scrBuffer
	res	6,b					;; BC = posicion en pantalla (conmuta segun el buffer de pantalla actual, set/res)

	ld	a,COIN_N_TILES_W			;; A' = W
	ex	af,af'
	ld	a,COIN_N_TILES_H			;; A  = H

	pop	de					;; DE = posicion del tile de la esquina superior izquierda en DECOMP_TILE_MAP

	call	addDirtyTilesRect
	jp	addDirtyTilesRect_doubleBuffer

	;; DATOS ---------------------------------------------------------

			;; DATOS FIJOS DE LAS MONEDAS

coin_fixed_info		;; de sprite
			db	0				;;     = draw mode
			dw	coin_frames			;; +2  = frames
			dw	coin_anims			;; +4  = animaciones

			db	7				;; +10 = ancho	(0 = 16, #10 = 20, ...)
			db	16				;; +11 = alto en pixeles
			db	2				;; +12 = ancho en tiles
			db	0				;; +13 = pivotX
			db	0				;; +14 = pivotY
			;; de actor
			db	-3				;; +34 = actor_collisionbb_xr_off
			db	4				;; +35 = actor_collisionbb_yr_off
			db	4				;; +36 = actor_collisionbb_w_off
			db	8				;; +37 = actor_collisionbb_h_off

			;; ANIMACIONES

COIN_ROTATION_ANIM_ID	EQU	0

			ALIGN 	2

coin_anims		dw	coin_rotation_anim		;; anim id 0 -> volando derecha
			; ...

coin_rotation_anim	db	6				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0, 2, 4, 2, 6, 8		;; frame id (multiplo de 4)

			ALIGN	2

coin_frames		dw	coin_frame0
			dw	coin_frame1
			dw	coin_frame2
			dw	coin_frame3
			dw	coin_frame4

;			ALIGN	4

coin_frame0
			db	0 * 4 + COIN_BASE_TILE + 0,	0 * 4 + COIN_BASE_TILE + 1
			db	0 * 4 + COIN_BASE_TILE + 2,	0 * 4 + COIN_BASE_TILE + 3

coin_frame1
			db	1 * 4 + COIN_BASE_TILE + 0,	1 * 4 + COIN_BASE_TILE + 1
			db	1 * 4 + COIN_BASE_TILE + 2,	1 * 4 + COIN_BASE_TILE + 3

coin_frame2
			db	2 * 4 + COIN_BASE_TILE + 0,	2 * 4 + COIN_BASE_TILE + 1
			db	2 * 4 + COIN_BASE_TILE + 2,	2 * 4 + COIN_BASE_TILE + 3

coin_frame3
			db	3 * 4 + COIN_BASE_TILE + 0,	3 * 4 + COIN_BASE_TILE + 1
			db	3 * 4 + COIN_BASE_TILE + 2,	3 * 4 + COIN_BASE_TILE + 3

coin_frame4
			db	4 * 4 + COIN_BASE_TILE + 0,	4 * 4 + COIN_BASE_TILE + 1
			db	4 * 4 + COIN_BASE_TILE + 2,	4 * 4 + COIN_BASE_TILE + 3

coin_frame_blank	;; vacio
			db	0, 0
			db	0, 0
