
	;; =========================================================================================
	;; DEMO DE MOTOR DE SCROLL HORIZONTAL CON PRECISION DE PIXEL
	;; =========================================================================================

	;; * El scroll cambia el hsync (CRTC R3) para tener precision de byte y se combina con 
	;;   cambios en la paleta de colores para alcanzar precision de pixel en mode 0.

	;; * Al cambiar el hsync, hace que solo sea compatible con el monitor del CPC o CRT TVs.

	;; * Los cambios de paleta limita a usar solo 4 colores para el fondo x4 franjas verticales de
	;;   pantalla (cada franja con color de fondo + negro + 2 colores propios). Los sprites tambien
	;;   sufren limitaciones, quedando la distribucion de la paleta asi:

	;;   color  0 -> fondo (comun a todas las franjas del fondo y sprites)
	;;   color  1 -> negro (comun a todas las franjas del fondo y sprites)
	;;   color  2 -> tono oscuro (comun a fondo y sprites pero cambia en cada franja del fondo)
	;;   color  3 -> tono claro (comun a fondo y sprites pero cambia en cada franja del fondo)
	;;   color  4 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  5 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  6 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  7 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  8 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color  9 -> exclusivo del fondo (cambia en cada franja del fondo)
	;;   color 10 -> exclusivo del fondo (comun a todas las franjas del fondo)
	;;   color 11 -> exclusivo del fondo (comun a todas las franjas del fondo)
	;;   color 12 -> exclusivo sprites
	;;   color 13 -> exclusivo sprites
	;;   color 14 -> exclusivo sprites
	;;   color 15 -> exclusivo sprites

	;; * Los gfxs de los tiles tienen limitaciones que se explican en tileMapSupport.asm

	;; * Para poder cambiar la paleta en cada franja de pantalla en el momento justo, es necesario
	;;   que las interrupciones esten activadas en todo momento.

	;; * Se usa doble buffer para eliminar posibles distorsiones o parpadeos en los sprites.

	;; * Para agregar una rutina por interrupciones a 50hz, llamarla desde el lugar especificado
	;;   dentro de myInt0


	;; - RESUMEN DE USO DE MEMORIA:

	;; #0000 - #0037 				= pila
	;; #0100 - end_of_actor_buffers (#0aff aprox) 	= buffers varios (pueden crecer hasta #fff)
	;; #1000 - #17ff				= buffer de mapeado descomprimido (pantalla actual)
	;; #1800 - #3fff 				= codigo, datos, recursos... (10.2kb)
	;; #4000 - #4FE7				= pantalla del marcador
	;; #4fe8 - #7fff				= codigo, datos, recursos... (12.3kb)
	;; #8000 - #bfff				= primer buffer de pantalla
	;; #c000 - #ffff				= segundo buffer de pantalla

	;; * Descontando todos los buffers y areas de pantalla, quedan libres casi 24kb en los
	;;   primeros 64kb de memoria, por lo que juegos sencillos para 64kb son factibles.


	;; - NOTAS SOBRE OPTIMIZACION:

	;; * El motor esta pensado para juegos de estilo grafico minimalista con mucho color liso 
	;;   en el fondo. Tanto el scroll como el dibujado de sprites aprovecha esta cualidad para
	;;   mejorar el rendimiento.

	;; * Siempre que sea posible usar animaciones de tiles en vez de sprites, ya que estas no
	;;   necesitan borrarse y redibujarse en cada frame y son mas rapidas. Las monedas de esta
	;;   demo usan animacion de tiles para poder poner muchas a la vez en pantalla. Un fx de
	;;   desaparicion de la moneda cuando la coges podria implementarse tambien con animacion
	;;   de tiles de forma mas eficiente que como sprite.

	;; * Una posible optimizacion de velocidad es implementar el dibujado de tiles compilados
	;;   al menos en los tiles sin colision (y si la memoria lo permite)


	;; - OPCIONES QUE REQUIEREN CAMBIOS EN EL MOTOR:

	;; * Si se renuncia a tener precision de pixel, todos los colores de la paleta quedan 
	;;   libres para usarlos como se quiera y se podrian apagar las interrupciones y usar la pila
	;;   para dibujar tiles y sprites mas rapido. Entonces los sprites que se muevan tambien con 
	;;   precision de byte pueden ahorrarse los frames en pixeles impares para ahorrar memoria.


	;; - TODO:

	;; * Cuando se dibuja una pantalla completa del mapeado para empezar por un lugar
	;;   determinado, crear tambien los enemigos e items que correspondan.
	;;   (metodo setMapScreen en tileMapSupport.asm)

	;; * Explotar mas las animaciones de tiles para implementar cosas como plataformas moviles
	;;   (animando solo los tiles justos en cada movimiento)


	;; CONFIGURACION DE ENSAMBLADO ===============================

	WRITE DIRECT -1,-1,#c0

	run	start;, break

	LET	SAVE_BIN 	= 0		;; activar para generar ejecutable
	LET	DEBUG 		= 0		;; activar para ver barra de consumo de CPU
	LET	ONE_FRAME 	= 1		;; activar solo si va a 50 fps para optimizar

	NOLIST

	;; MACROS =====================================================

	read "macros\MACRO_setCRTCReg.asm"
	read "macros\MACRO_waitCycles.asm"
	read "macros\MACRO_code.asm"
	read "macros\MACRO_orgf.asm"

	;; Pone el border de un color

	MACRO	BORDER_COLOR	_value	;; 14c

	ld	bc,#7f10
	out	(c),c
	ld	c,_value
	out	(c),c

	MEND

	;; CONSTANTES ================================================

	read "screen_h.asm"

	read "HSEngine\tileMapSupport_h.asm"
	read "HSEngine\scrollSupport_h.asm"
	read "HSEngine\sprite_h.asm"
	read "HSEngine\actor_h.asm"
	read "HSEngine\actorMng_h.asm"
	read "HSEngine\enemy_h.asm"

	read "bird_h.asm"
	read "turtle_h.asm"
	read "coin_h.asm"
	read "player_h.asm"

	read "util\keyboard_h.asm"
	read "util\colors_h.asm"


BOTTOM_BORDER_COLOR		EQU	COLOR_AZUL
BG_COLOR			EQU	COLOR_AZUL_CIELO
MY_COLOR_01			EQU	COLOR_NEGRO

				LIST

				;; direcciones de memoria

MY_STACK_LEN			EQU	#38
myStack				EQU	0 + MY_STACK_LEN
dirty_tiles_rect0		EQU	#100 + DIRTY_TILES_RECT_LEN			;; buffer 0 de rectangulos de tiles sucios para borrar (puntero al final de la lista) (128 bytes)
dirty_tiles_rect1		EQU	dirty_tiles_rect0 + DIRTY_TILES_RECT_LEN	;; buffer 1 de rectangulos de tiles sucios para borrar (puntero al final de la lista) (128 bytes)
tilesAdr_l			EQU	dirty_tiles_rect1				;; buffer para punteros de tiles para optimizacion de velocidad al dibujarlos (byte bajo) (256 bytes)
tilesAdr_h			EQU	tilesAdr_l + #100				;; buffer para punteros de tiles para optimizacion de velocidad al dibujarlos (byte alto) (256 bytes)
sprite_msk_table		EQU	tilesAdr_h + #100				;; buffer con mascaras para impresion de sprites (256 bytes)
;sprite_flip_table		EQU	sprite_msk_table + #100				;; buffer con bytes de gfxs invertidos horizontalmente (256 bytes)
sprite_addy_table		EQU	sprite_msk_table + #100				;; buffer con direcciones de memoria de cada scanline (160 * 2 = 320 bytes)
actors_buffer			EQU	GAMEAREA_H_PIXEL * 2 + sprite_addy_table

				;; buffers para actors

myBirdsInfo			EQU	actors_buffer
myTurtlesInfo			EQU	N_BIRDS_MAX * BIRD_INFO_LEN + actorMng_offset + myBirdsInfo
myCoinsInfo			EQU	N_TURTLES_MAX * TURTLE_INFO_LEN + actorMng_offset + myTurtlesInfo
				;...
;end_of_actor_buffers		EQU	N_COINS_MAX * COIN_INFO_LEN + actorMng_offset + myCoinsInfo

DECOMP_TILE_MAP			EQU	#1000						;; buffer del mapeado de tiles descomprimido (pantalla actual) (debe estar en una direccion multiplo de #1000)
CODE_INI			EQU	DECOMP_TILE_MAP + DECOMP_TILE_MAP_SIZE		;; inicio del codigo

				NOLIST

	;; CODIGO =====================================================

	LIST

	ORG	CODE_INI

start	;; AQUI	EMPIEZA LA EJECUCION

	NOLIST

        di
	ld	sp,myStack

	;; pone la pantalla azul

	call	wVb
	ld	bc,#7f8c + 2
	ld	de,#0100 + COLOR_AZUL
	out	(c),c
	out	(c),d
	out	(c),e

	call	initGame

	;       Initializes the player.
	;       HL = music address.
	ld	hl,myMusic
	call	PLY_AKY_Init

	IF	ONE_FRAME

	call	keyscan

	ENDIF

	;; Loop principal

	call	wVb1
mainLoop
	call	wVb2

;	call	wVb

	di

	;; OJO!, momento critico antes de que salte la int0 (no cambiar de lugar)

_counterFrame
	ld	hl,0				;; incrementa el contador de frame
	inc	hl
	ld	(_counterFrame + 1),hl

	;; procesa el scroll
	call	processScroll

	ei

	IF	DINAMIC_PALETTE_CHANGES

	call	refreshMapColors

	ENDIF

	;; resto de procesos por cada frame

	IFNOT	ONE_FRAME

	call	keyscan

	ENDIF

	;; borra los sprites anteriores

	call	clearDirtyTilesRect

	;; procesa el jugador
	call	player_process

	ld	bc,#0838			;; cachea estos valores en los registros alternativos porque son comunes a todas
	ld	de,NEXT_SCANLINE_MOD		;; la rutinas de pintado
	exx

	;; procesa otros actores del juego

	;; monedas
	call	coin_processAll

	;; pajaros
	ld	ix,myBirdsInfo		; IX = puntero a lista de actors
	ld	hl,bird_process		; HL = puntero a la rutina process del actor
	call	processActorMng

	;; tortugas
	ld	ix,myTurtlesInfo	; IX = puntero a lista de actors
	ld	hl,turtle_process	; HL = puntero a la rutina process del actor
	call	processActorMng

	;; dibuja el sprite del jugador
	ld	ix,player_info
	call	drawSprite

	IF	DEBUG

	BORDER_COLOR	COLOR_MAGENTA

	ENDIF

;	WAIT_CYCLES	64*14			;; prueba de rendimiento

	CHECK_ESC_KEY
	jr	nz, mainLoop

	;; Finalizacion -------------------------------------------

	call	toStandarScreen

	;; hace reset

	ld 	hl,rst0
	ld 	bc,5
	ld 	d,b
	ld 	e,b
	ldir
	rst 	0
rst0
	ld 	bc,#7F89
	out 	(c),c

;	jr	$

	;; Inicializacion -----------------------------------------

initGame
	;; dibuja una pantalla completa del mapeado

	ld	hl,0
	ld	(_counterFrame + 1),hl

	;; sprites
	call	initSprites

	;; scroll
	ld	ix,player_info
	call	initScroll

	;; mapeado

	ld	de,tileMap
	ld	bc,tileMap_len
	ld	hl,tiles
	call 	initMap

	;; jugador
	call	player_init

	;; enemigos
	call	enemy_initSupport

	;; pone pantalla con split
	call	toSplitScreen

	;; pone los colores de los sprites
	ld	hl,sprite_colors	;; HL = lista de colores
	ld	a,12			;; A = primer color
	ld	c,4			;; C = numero de colores
	call	setPalette

	; ...

	ret

	;; Cambia del modo de pantalla standar a modo split ----------

toSplitScreen
	di
        ld 	hl,#C9FB
        ld 	(#38),hl

	call	wVb

	SET_CRTC_REG	3,R3_NO_SCROLL_VALUE

	IF	DEBUG
;	ld	hl,#3132			;; H = x, L = w	-> borde por la izq
	ld	hl,#3732			;; H = x, L = w	-> borde por la der
	ELSE
	ld	hl,#3432			;; H = x, L = w
	ENDIF
	ld	de,#1814			;; D = y, E = h
	call	redim_screen

	ld	a,#C3				;; interrupciones
	ld	(#38),a
	ld	hl,myInt0
	ld	(#39),hl

	ret

	;; Cambia del modo de pantalla con split al modo standar ------

toStandarScreen
	call	wVb

	di

	SET_CRTC_REG	12,%00110000		;; pantalla en #c000
	SET_CRTC_REG	13,0

	SET_CRTC_REG	9,7			;; caracter de 8 pixeles de alto

        LD 	hl,#C9FB
        LD 	(#38),hl

	ld	hl,#2e28			;; H = x, L = w
	ld	de,#1e19			;; D = y, E = h
	jp	redim_screen


	;; Interrupciones =====================================================

	;; * La unica interrupcion que tiene cierta tolerancia al retraso es la int0

	;; Int 0 --------------------------------------------------------------

	;; Inicializa colores del marcador

	;; VCC = #18
	;; VLC = #02

myInt0
	push	af
	push	bc
;	push	de
	push	hl

	;; hsync standar (REG 3)

	SET_CRTC_REG	3, R3_NO_SCROLL_VALUE

	;; pone los colores del marcador

	ld	hl,scoreboard_colors	;; HL = lista de colores
	xor	a			;; A = primer color
	ld	c,4			;; C = numero de colores
	call	setPalette

	BORDER_COLOR	BG_COLOR

	;; llamar a la rutina de la musica aqui (guardar los registros que falten en la pila)

	push	de
	exx
	push	bc
	push	de
	push	hl
	ex	af,af'
	push	af

	call	PLY_AKY_Play

	pop	af
	ex	af,af'
	pop	hl
	pop	de
	pop	bc
	exx
	pop	de

	; ...

	ld	hl,myInt1
	ld	(&39),hl

	pop	hl
;	pop	de
	pop	bc
	pop	af

	ei
	ret

	;; Int 1 --------------------------------------------------------------

	;; Define el area de pantalla del marcador

	;; VCC = #1e
	;; VLC = #06

myInt1
	push	af
	push	bc

	ld	bc,myInt2
	ld	(&39),bc

	SET_CRTC_REG	7,63				;; evita un vsync antes de tiempo
	SET_CRTC_REG	6,SCOREBOARD_H			;; tamanyo de pantalla del marcador
	SET_CRTC_REG	1,SCOREBOARD_W
	SET_CRTC_REG	5,0

	ld	bc,#7f8c + SCOREBOARD_SCREEN_MODE	;; modo de pantalla del marcador
	out	(c),c

	WAIT_CYCLES	52;38+28-14			;; espera 6 scanlines (el equivalente al +3 que se le da al r4 al final del halt 4)

	SET_CRTC_REG	9,1 				;; caracter de 2 pixeles de alto

	SET_CRTC_REG	12, SCOREBOARD_SCREEN_R12	;; direccion de pantalla
	SET_CRTC_REG	13, SCOREBOARD_SCREEN_R13

	SET_CRTC_REG	4,31 + 4 			;; (deja seleccionado el reg4)
	
	pop	bc
	pop	af

	ei
	ret

	;; Int 2 --------------------------------------------------------------

	;; Define el area de pantalla del juego

	;; VCC = #14
	;; VLC = #00

myInt2
	push	af
	push	bc

	ld	bc,#bd00 + 21			;; r4
	out	(c),c

	SET_CRTC_REG	5, 8-4			;; ajuste vertical

	ld	bc,#bc03			;; ajusta el hsync para precision de byte del scroll horizontal
	out	(c),c
_reg3_value
	ld	bc,#bd00 + R3_NO_SCROLL_VALUE
	out	(c),c

	push	de
_game_screen_crtc_adr
	ld	de,0

	ld	bc,#bc00+12			;; direccion de pantalla
	out	(c),c
	inc	b
	out	(c),d
	dec	b
	inc	c
	out	(c),c
	inc	b
	out	(c),e

	push	hl

	ld	hl,myInt3
	ld	(&39),hl

_game_common_colors_bar0
	ld	de,COLOR_BLANCO * #100 + COLOR_AZUL_PASTEL	;; D = tono claro, E = tono oscuro
_game_common_colors
	ld	hl,MY_COLOR_01 * #100 + BG_COLOR		;; H = outline,	  L = fondo

	SET_CRTC_REG	7,24

	; si fuese necesario, aqui se puede poner algunos nops
	; ...

	ld	bc,#7f8c + GAMEAREA_SCREEN_MODE
	out	(c),c

;	SET_CRTC_REG	6,GAMEAREA_H
	SET_CRTC_REG	1,GAMEAREA_W
	SET_CRTC_REG	9,7					;; caracter de 8 pixeles de alto

	ld	bc,#7f0b					;; cambio de los colores del 2 al 11 para el scroll
_bar0_hScroll
	call	setColors_bar0_l

	pop	hl
	pop	de
	pop	bc
	pop	af

	ei
	ret

	;; Int 3 --------------------------------------------------------------

	;; Cambio de colores para el scroll

	;; VCC = #04
	;; VLC = #06

myInt3
	push	af
	push	bc
	push	de
	push	hl

	SET_CRTC_REG	4,#7f

_game_common_colors_bar1
	ld	de,COLOR_NARANJA * #100 + COLOR_ROJO	;; D = tono claro, E = tono oscuro
	ld	hl,(_game_common_colors + 1)		;; H = outline,	  L = fondo
	ld	bc,#7f09				;; colores del 2 al 9
_bar1_hScroll
	call	setColors_bar_l

	ld	hl,myInt4
	ld	(&39),hl

	pop	hl
	pop	de
	pop	bc
	pop	af

	ei
	ret

	;; Int 4 --------------------------------------------------------------

	;; Cambio de colores para el scroll

	; VCC = #0b
	; VLC = #02

myInt4
	push	af
	push	bc
	push	de
	push	hl

	ld	hl,myInt5
	ld	(&39),hl

	BORDER_COLOR	BOTTOM_BORDER_COLOR

	IF	ONE_FRAME

	call	keyscan						;; si va en un frame aprovecha para leer el teclado aqui y ahorrar unos ciclos
								;; si no va en un frame no conviene leer el teclado aqui porque se leeria mas veces de las necesarias
	WAIT_CYCLES	256-215					;; espera para hacer el cambio de colores en la scanline exacta

	ELSE

	WAIT_CYCLES	256					;; espera para hacer el cambio de colores en la scanline exacta

	ENDIF

_game_common_colors_bar2
	ld	de,COLOR_VERDE_BRILLANTE * #100 + COLOR_VERDE	;; D = tono claro, E = tono oscuro
	ld	hl,(_game_common_colors + 1)			;; H = outline,	  L = fondo
	ld	bc,#7f09					;; colores del 2 al 9
_bar2_hScroll
	call	setColors_bar_l

	pop	hl
	pop	de
	pop	bc
	pop	af

	ei
	ret

	;; Int 5 --------------------------------------------------------------

	;; Cambio de colores para el scroll

	;; VCC = #11
	;; VLC = #06

myInt5
	push	af
	push	bc
	push	de
	push	hl

	ld	hl,myInt0
	ld	(&39),hl

	WAIT_CYCLES	12					;; espera para hacer el cambio de colores en la scanline exacta

_game_common_colors_bar3
	ld	de,COLOR_CYAN_PASTEL * #100 + COLOR_CYAN	;; D = tono claro, E = tono oscuro
	ld	hl,(_game_common_colors + 1)			;; H = outline,	  L = fondo
	ld	bc,#7f09					;; colores del 2 al 9
_bar3_hScroll
	call	setColors_bar_l

	pop	hl
	pop	de
	pop	bc
	pop	af

	ei
	ret

	;; Establece los colores de una zona de la pantalla del juego (ajustado a la izq)

	;; B = #7f
	;; C = color mas alto de lo que cambiara
	;; H = outline,	  L = bg
	;; D = tono claro, E = tono oscuro

	;; (74+18 = 92c)

setColors_bar0_l
	out	(c),c			;; color 11
	out	(c),l			;; bg
	dec	c			
	out	(c),c			;; color 10
	out	(c),h			;; outline
	dec	c

setColors_bar_l
	out	(c),c			;; color 9
	out	(c),d			;; tono claro
	dec	c			
	out	(c),c			;; color 8
	out	(c),e			;; tono oscuro
	dec	c			
	out	(c),c			;; color 7
	out	(c),h			;; outline
	dec	c			
	out	(c),c			;; color 6
	out	(c),h			;; outline
	dec	c			
	out	(c),c			;; color 5
	out	(c),d			;; tono claro
	dec	c			
	out	(c),c			;; color 4
	out	(c),e			;; tono oscuro
	dec	c			
	out	(c),c			;; color 3
	out	(c),d			;; tono claro
	dec	c			
	out	(c),c			;; color 2
	out	(c),e			;; tono oscuro
	ret

	;; Establece los colores de una zona de la pantalla del juego (ajustado a la der)

	;; B = #7f
	;; C = color mas alto de lo que cambiara
	;; H = outline,	  L = bg
	;; D = tono claro, E = tono oscuro

	;; (74+18 = 92c)

setColors_bar0_r
	out	(c),c			;; color 11	; 4c
	out	(c),h			;; outline	; 4c
	dec	c					; 1c
	out	(c),c			;; color 10	; 4c
	out	(c),l			;; bg		; 4c
	dec	c					; 1c

setColors_bar_r
	out	(c),c			;; color 9	; 4c
	out	(c),e			;; tono oscuro	; 4c
	dec	c					; 1c
	out	(c),c			;; color 8	; 4c
	out	(c),d			;; tono claro	; 4c
	dec	c					; 1c
	out	(c),c			;; color 7	; 4c
	out	(c),d			;; tono claro	; 4c
	dec	c					; 1c
	out	(c),c			;; color 6	; 4c
	out	(c),e			;; tono oscuro	; 4c
	dec	c					; 1c
	out	(c),c			;; color 5	; 4c
	out	(c),h			;; outline	; 4c
	dec	c					; 1c
	out	(c),c			;; color 4	; 4c
	out	(c),h			;; outline	; 4c
	dec	c					; 1c
	out	(c),c			;; color 3	; 4c
	out	(c),d			;; tono claro	; 4c
	dec	c					; 1c
	out	(c),c			;; color 2	; 4c
	out	(c),e			;; tono oscuro	; 4c
	ret						; 3c

	;; MAS CODIGO ====================================================

	;; librerias

;	read "util\getCRTCType.asm"
	read "util\setCRTC.asm"
	read "util\setPalette.asm"
	read "util\waitVBlank.asm"
	read "util\redimScr.asm"
	read "util\keyboard.asm"
	read "util\fillMem.asm"

	;; motor
	read "HSEngine\scrollSupport.asm"
	read "HSEngine\tileMapSupport.asm"
	read "HSEngine\sprite.asm"
	read "HSEngine\actor.asm"
	read "HSEngine\actorMng.asm"
	read "HSEngine\enemy.asm"

	;; demo
	read "player.asm"
	read "bird.asm"
	read "turtle.asm"
	read "coin.asm"

	;; Procesa un codigo de tecla extendido
	;; (justo antes debe haberse comprobado la tecla por la via normal, tipo CHECK_ESC_KEY)

	;; DE = puntero a la variable extendida de la tecla

;processExtendedKeyCode				; 13 bytes
;	scf
;	jr	z,$+3
;	ccf
;	ld	a,(de)
;	rla
;	and	KEEP_PRESSED_KEYCODE
;	ld	(de),a
;	ret

	;; codigos de teclas extendidos (0 = no pulsado, 1 = recien pulsado, 2 = recien soltado, 3 = mantenido pulsado)

;left_keyCode_ex	db	0
;right_keyCode_ex	db	0

	;; DATOS ========================================================

			;; COLORES USADOS

scoreboard_colors	db 	BG_COLOR
			db	&54
			db	&4c
			db	&4b
sprite_colors		db	COLOR_AZUL_BRILLANTE
			db	COLOR_ROJO_BRILLANTE
			db	COLOR_ROSA
			db	COLOR_BLANCO

			;; IDs de enemigos

BIRD_HI_ID		EQU	2				;; pajaro que vuela alto
BIRD_LOW_ID		EQU	4				;; pajaro que vuela un poco mas bajo
TURTLE_HI_ID		EQU	6				;; tortuga que aparece sobre una tuberia baja
TURTLE_LOW_ID		EQU	8				;; tortuga que aparece en el suelo
COIN_ROW00_ID		EQU	10				;; moneda en la fila de tiles 0
COIN_ROW02_ID		EQU	12				;; moneda en la fila de tiles 2
COIN_ROW04_ID		EQU	14				;; moneda en la fila de tiles 4
COIN_ROW06_ID		EQU	16				;; moneda en la fila de tiles 6
COIN_ROW08_ID		EQU	18				;; moneda en la fila de tiles 8
COIN_ROW10_ID		EQU	20				;; moneda en la fila de tiles 10
COIN_ROW12_ID		EQU	22				;; moneda en la fila de tiles 12
COIN_ROW14_ID		EQU	24				;; moneda en la fila de tiles 14
			; ...

			ALIGN	2

			;; Indices de constructores de los diferentes tipos de enemigos

createEnemList		dw	0			; +0
			dw	bird_hi_create		; +2	pajaritos volando alto
			dw	bird_low_create		; +4	pajaritos volando bajo
			dw	turtle_hi_create	; +6	tortugas
			dw	turtle_low_create	; +8	tortugas
			dw	coin_row00_create	; +10
			dw	coin_row02_create	; +12
			dw	coin_row04_create	; +14
			dw	coin_row06_create	; +16
			dw	coin_row08_create	; +18
			dw	coin_row10_create	; +20
			dw	coin_row12_create	; +22
			dw	coin_row14_create	; +24
			; ...



	;; RECURSOS ========================================================

				;; Recorridos precalculados

small_sin_route			incbin "resources\small_sin_route.bin"

				;; GFXS

				ALIGN	4

spr_mario_stop_par		incbin "resources\spr_mario_stop_par.bin"
spr_mario_stop_impar		incbin "resources\spr_mario_stop_impar.bin"

spr_mario_walk01_par		incbin "resources\spr_mario_walk01_par.bin"
spr_mario_walk01_impar		incbin "resources\spr_mario_walk01_impar.bin"

spr_mario_walk02_par		incbin "resources\spr_mario_walk02_par.bin"
spr_mario_walk02_impar		incbin "resources\spr_mario_walk02_impar.bin"

spr_mario_walk03_par		incbin "resources\spr_mario_walk03_par.bin"
spr_mario_walk03_impar		incbin "resources\spr_mario_walk03_impar.bin"

spr_mario_walk04_par		incbin "resources\spr_mario_walk04_par.bin"
spr_mario_walk04_impar		incbin "resources\spr_mario_walk04_impar.bin"

spr_mario_stop_i_par		incbin "resources\spr_mario_stop_i_par.bin"
spr_mario_stop_i_impar		incbin "resources\spr_mario_stop_i_impar.bin"

spr_mario_walk01_i_par		incbin "resources\spr_mario_walk01_i_par.bin"
spr_mario_walk01_i_impar 	incbin "resources\spr_mario_walk01_i_impar.bin"

spr_mario_walk02_i_par		incbin "resources\spr_mario_walk02_i_par.bin"
spr_mario_walk02_i_impar 	incbin "resources\spr_mario_walk02_i_impar.bin"

spr_mario_walk03_i_par		incbin "resources\spr_mario_walk03_i_par.bin"
spr_mario_walk03_i_impar	incbin "resources\spr_mario_walk03_i_impar.bin"

spr_mario_walk04_i_par		incbin "resources\spr_mario_walk04_i_par.bin"
spr_mario_walk04_i_impar	incbin "resources\spr_mario_walk04_i_impar.bin"


				;; marcador

				ORGF	SCOREBOARD_SCREEN_ADR, 0

				incbin "resources\mario_sb.bin"		;; PROVISIONAL

				;; mapeado 

				read "map.asm"


				;; MAS GFXS

				;; pajaro

				ALIGN	4

spr_bird_fly01_par		incbin "resources\spr_bird_fly01_par.bin"
spr_bird_fly01_impar		incbin "resources\spr_bird_fly01_impar.bin"

spr_bird_fly02_par		incbin "resources\spr_bird_fly02_par.bin"
spr_bird_fly02_impar		incbin "resources\spr_bird_fly02_impar.bin"

spr_bird_fly03_par		incbin "resources\spr_bird_fly03_par.bin"
spr_bird_fly03_impar		incbin "resources\spr_bird_fly03_impar.bin"

spr_bird_fly01_i_par		incbin "resources\spr_bird_fly01_i_par.bin"
spr_bird_fly01_i_impar 		incbin "resources\spr_bird_fly01_i_impar.bin"

spr_bird_fly02_i_par		incbin "resources\spr_bird_fly02_i_par.bin"
spr_bird_fly02_i_impar 		incbin "resources\spr_bird_fly02_i_impar.bin"

spr_bird_fly03_i_par		incbin "resources\spr_bird_fly03_i_par.bin"
spr_bird_fly03_i_impar		incbin "resources\spr_bird_fly03_i_impar.bin"

				;; tortuga

				ALIGN	4

spr_turtle_stop_par		incbin "resources\spr_turtle_walk01_par.bin"
spr_turtle_stop_impar		incbin "resources\spr_turtle_walk01_impar.bin"

spr_turtle_walk01_par		incbin "resources\spr_turtle_walk02_par.bin"
spr_turtle_walk01_impar		incbin "resources\spr_turtle_walk02_impar.bin"

spr_turtle_walk02_par		incbin "resources\spr_turtle_walk03_par.bin"
spr_turtle_walk02_impar		incbin "resources\spr_turtle_walk03_impar.bin"

spr_turtle_walk03_par		incbin "resources\spr_turtle_walk04_par.bin"
spr_turtle_walk03_impar		incbin "resources\spr_turtle_walk04_impar.bin"

spr_turtle_walk04_par		incbin "resources\spr_turtle_walk05_par.bin"
spr_turtle_walk04_impar		incbin "resources\spr_turtle_walk05_impar.bin"

spr_turtle_stop_i_par		incbin "resources\spr_turtle_walk01_i_par.bin"
spr_turtle_stop_i_impar		incbin "resources\spr_turtle_walk01_i_impar.bin"

spr_turtle_walk01_i_par		incbin "resources\spr_turtle_walk02_i_par.bin"
spr_turtle_walk01_i_impar 	incbin "resources\spr_turtle_walk02_i_impar.bin"

spr_turtle_walk02_i_par		incbin "resources\spr_turtle_walk03_i_par.bin"
spr_turtle_walk02_i_impar 	incbin "resources\spr_turtle_walk03_i_impar.bin"

spr_turtle_walk03_i_par		incbin "resources\spr_turtle_walk04_i_par.bin"
spr_turtle_walk03_i_impar	incbin "resources\spr_turtle_walk04_i_impar.bin"

spr_turtle_walk04_i_par		incbin "resources\spr_turtle_walk05_i_par.bin"
spr_turtle_walk04_i_impar	incbin "resources\spr_turtle_walk05_i_impar.bin"


	; music

	read 	"util\PlayerAky_CPC.asm"
	read	"resources\music.aky_playerconfig.asm"
myMusic	read	"resources\music.aky.asm"

	LIST

	;; END

__len	EQU	$-start

	NOLIST

