
	;; =========================================================================================
	;; JUGADOR
	;; =========================================================================================

	;; * Hereda de la estructura de datos de actor

	;; CODIGO ========================================================

	;; Metodos principales:

	;; * player_init		-> Inicializacion del jugador
	;; * player_process 		-> Process del jugador
	;; * player_on_tile_collision	-> Callback de colision con tiles
	;; * player_on_coin_collision	-> Callback de colision con monedas
	;; * checkPlayerActorCollision	-> Comprueba colision de un actor con el jugador

	;; ----------------------------------------------------------------

	;; Inicializacion

player_init
	ld	ix,player_info
	call	actor_init

	xor	a
	ld	(_player_h_moved + 1),a

	ld	a,MARIO_WALK_RIGHT_ANIM_ID	; A = anim
	ld	b,0				; B = sec
	jp	setSpriteAnimSec

	;; Process del jugador

	;; Entrada:
	;; 	IX = estructura de datos del jugador

player_process
	ld	ix,player_info

	call	player_process_jump

	call	player_move_h

	call	player_process_animation

	call	actor_process_gravity_motion
	call	c, player_on_tile_collision

	;; establece los valores para colisiones

	ld	a,(ix + actor_collisionbb_x1_off)
	ld	(_collision_x1 + 1),a
	ld	a,(ix + actor_collisionbb_x2_off)
	ld	(_collision_x2 + 1),a
	ld	a,(ix + actor_collisionbb_y1_off)
	ld	(_collision_y1 + 1),a
	ld	a,(ix + actor_collisionbb_y2_off)
	ld	(_collision_y2 + 1),a

	ret

	;; Mueve el jugador izq/der

player_move_h
	ld	d,0
	CHECK_CURSOR_LEFT_KEY
	jr	nz, $+3
	dec	d
	CHECK_CURSOR_RIGHT_KEY
	jr	nz, $+3
	inc	d

	ld	e,(ix + actor_speed_off)

	ld	a,(_player_h_moved + 1)
	or	a
	ld	a,d
	jr	z, player_h_stopped
	jp	m, player_moving_left

	;; yendo a la derecha ------------------

	cp	-1
	jr	nz, player_no_changing_left

	;; cambia a la izq

player_left_deceleration
	dec	e				;; cambio de direccion a velocidad mas rapida
	dec	e
	jp	p, player_h_mov_continue
	xor	a
	jr	player_h_mov_continue2

player_no_changing_left
	cp	1

	IF	PLAYER_CONTINUOUS_MOVEMENT

	jr	nz, player_h_mov_continue	;; para movimiento continuo

	ELSE

	jr	nz, player_left_deceleration

	ENDIF

	inc	e				;; aceleracion mas lenta

	ld	a,PLAYER_MAX_SPEED
	cp	e
	jr	nc, player_h_mov_continue
	jr	player_h_mov_continue2

	;; yendo a la izq ------------------

player_moving_left
	cp	1
	jr	nz, player_no_changing_right

	;; cambia a la der

player_right_deceleration
	inc	e				;; cambio de direccion a velocidad mas rapida
	inc	e
	jp	m, player_h_mov_continue

	xor	a
	jr	player_h_mov_continue2

player_no_changing_right
	cp	-1

	IF	PLAYER_CONTINUOUS_MOVEMENT

	jr	nz, player_h_mov_continue	;; para movimiento continuo

	ELSE

	jr	nz, player_right_deceleration

	ENDIF

	dec	e				;; aceleracion mas lenta

	ld	a,-PLAYER_MAX_SPEED
	cp	e
	jr	c, player_h_mov_continue
	jr	player_h_mov_continue2

	;; parado horizontalmente

player_h_stopped
	cp	-1
	jr	nz, player_no_start_left

	;; empieza movimiento izq

	xor	a
	ld	(player_animation_counter),a

	ld	a,-PLAYER_MIN_SPEED
	jr	player_h_mov_continue2

player_no_start_left
	cp	1
	jr	nz, player_h_mov_continue

	;; empieza movimiento derecha

	xor	a
	ld	(player_animation_counter),a

	ld	e,PLAYER_MIN_SPEED

player_h_mov_continue

	ld	a,e
player_h_mov_continue2

;	ld	a,8				;; * PRUEBA movimiento continuo pixel a pixel

	ld	(ix + actor_speed_off),a

	inc	a				;; redondeo
	repeat	PLAYER_INERTIA_BITS
	sra	a
	rend

	ld	(_player_h_moved + 1),a		;; guarda la info de hacia donde movio el jugador para la animacion

	;; Entrada:
	;;	IX = puntero al actor
	;;	 A = pixeles de desplazamiento
	;; Salida:
	;;	Flag C = 1 si hay colision
	;;	     A = tipo de colision
	;;	    HL = puntero al ID del tile en que colision

	call	actor_process_h_motion
	jr	nc, player_no_h_collision

	;; colision horizontal del jugador

	call	player_on_tile_collision

	ld	(ix + actor_speed_off),0	;; para el jugador

	ld	a,(ix + actor_state_off)
	cp	ACTOR_STATE_WALKING
	jr	nz,$+6
	ld	(ix + actor_state_off),ACTOR_STATE_STOPPED

	jr	player_h_collisioned

player_no_h_collision

	;; sin colision horizontal del jugador

	ld	a,(ix + actor_state_off)
	cp	ACTOR_STATE_STOPPED
	jr	nz, $+6
	ld	(ix + actor_state_off),ACTOR_STATE_WALKING

player_h_collisioned

	ret


	; Procesa la animacion

player_process_animation

_player_h_moved
	ld	a,0
	or	a
	jr	z, player_stopped

	;; andando

	ld	c,MARIO_WALK_RIGHT_ANIM_ID
	jp	p, $+7
	inc	c
	inc	c
	neg

	ld	d,a				;; D = velocidad de animacion
	ld	a,c

	cp	(ix + spr_anim_off)
	jr	nz, player_change_anim

player_animate
	ld	hl,player_animation_counter
	dec	(hl)
	ret	p

	ld	a,PLAYER_WALKING_ANIM_SPEED
	sub	d
	ld	(hl),a
	jp	nextSpriteSec

	;; parado

player_stopped
	ld	a,MARIO_STOP_RIGHT_ANIM_ID
	bit	1,(ix + spr_anim_off)
	jr	z, $+4
	add	2				;;  A = anim andando izq/der

	cp	(ix + spr_anim_off)
	ret	z

player_change_anim
	ld	b,0				;;  B = sec
	jp	setSpriteAnimSec


	; Procesa el salto

player_process_jump
	CHECK_CURSOR_UP_KEY
	ret	nz

	ld	a,(ix + actor_state_off)
	cp	ACTOR_STATE_JUMPING
	ret	z					;; si esta saltando no puede iniciar el salto

	ld	(ix + actor_state_off),ACTOR_STATE_JUMPING
	ld	(ix + actor_gravity_state_off),ACTOR_GRAVITY_STATE_GOING_UP	

	ld	(ix + actor_vertical_impulse_off),PLAYER_JUMP_SPEED AND #ff
	ld	(ix + actor_vertical_impulse_off + 1),PLAYER_JUMP_SPEED / #100

	ret

	; COLISIONES ----------------------------------------------------------

	;; Callback de colision con tiles

	;;	 A = tipo de colision
	;;	HL = puntero al ID del tile en que colision

player_on_tile_collision
	; ...
	ret

	;; Callback de colision con monedas

player_on_coin_collision
	; ...
	ret

	;; Comprueba colision de un actor con el jugador

	;; Entrada:
	;;	IX = info del actor
	;; 	_collision_x1, _collision_x2, _collision_y1, _collision_y2 deben estar previamente establecidas por el jugador

	;; Salida:
	;;	Flag C = 1 si hay colision

checkPlayerActorCollision

	ld	a,(ix + actor_collisionbb_x1_off)
_collision_x2
	cp	0
	ret	nc

	ld	a,(ix + actor_collisionbb_x2_off)
_collision_x1
	cp	0
	ccf
	ret	nc

	ld	a,(ix + actor_collisionbb_y1_off)
_collision_y2
	cp	0
	ret	nc

	ld	a,(ix + actor_collisionbb_y2_off)
_collision_y1
	cp	0
	ccf
	ret

	;; DATOS ========================================================

player_info		;; de sprite
			dw	0				;; +0  = x de pantalla
			db	0				;; +2  = y de pantalla
			db	SPRITE_DRAW_MODE_MSK		;; +3   = draw mode
			dw	mySprite_frames			;; +4  = frames
			dw	mySprite_anims			;; +6  = animaciones
			db	0				;; +8  = anim pos
			db	0				;; +9  = sec pos
			db	SPRITE_24W			;; +10  = ancho	(0 = 16, #10 = 20, ...)
			db	34				;; +11  = alto en pixeles
			db	24/8				;; +12 = ancho en tiles
			db	-5				;; +13 = pivotX
			db	-34				;; +14 = pivotY
			dw	0				;; +15 = current anim ptr 		(cache, lo escribe el codigo)
			dw	0				;; +17 = current frame ptr 		(cache)
			;; de actor
			db	ACTOR_STATE_STOPPED		;; +19 = estado
			db	ACTOR_GRAVITY_STATE_GOING_DOWN	;; +20 = estado de gravedad
			db	CAMERA_TRACKING_VALUE		;; +21 = flags
			dw	PLAYER_X_INI			;; +22 = actor_x_world_off
			dw	PLAYER_Y_INI			;; +24 = actor_y_world_off
			dw	0				;; +26 = actor_vertical_impulse_off
			dw	0				;; +28 = actor_speed_off
			dw	0				;; +30 = actor_route_x_off
			dw	0				;; +32 = actor_route_y_off
			db	-4				;; +34 = actor_collisionbb_xr_off
			db	-30				;; +35 = actor_collisionbb_yr_off
			db	8-2				;; +36 = actor_collisionbb_w_off
			db	29				;; +37 = actor_collisionbb_h_off
			db	0				;; +38 = actor_collisionbb_x1_off	(cache)
			db	0				;; +39 = actor_collisionbb_y1_off	(cache)
			db	0				;; +40 = actor_collisionbb_x2_off	(cache)
			db	0				;; +41 = actor_collisionbb_y2_off	(cache)
			db	0				;; +42 = actor_collisionbb_xt1_off	(cache)
			db	0				;; +43 = actor_collisionbb_yt1_off	(cache)
			db	0				;; +44 = actor_collisionbb_xt2_off	(cache)
			db	0				;; +45 = actor_collisionbb_yt2_off	(cache)
			db	0				;; +46 = actor_collisionbb_wt_off	(cache)
			db	0				;; +47 = actor_collisionbb_ht_off	(cache)
			;; de player
			db	0				;; +48 = player_lives_off
			db	0				;; +49 = player_coins_off

player_animation_counter	db	0

			;; ANIMACIONES

MARIO_STOP_RIGHT_ANIM_ID	EQU	0			;; parado mirando a la derecha
MARIO_STOP_LEFT_ANIM_ID		EQU	2			;; parado mirando a la izquierda
MARIO_WALK_RIGHT_ANIM_ID	EQU	4			;; andando/saltando derecha
MARIO_WALK_LEFT_ANIM_ID		EQU	6			;; andando/saltando izquierda

			ALIGN 	2

mySprite_anims		dw	mySprite_stop_right		;; anim id 0 -> parado derecha
			dw	mySprite_stop_left		;; anim 1d 2 -> parado izquierda
			dw	mySprite_walk_right		;; anim id 4 -> andando derecha
			dw	mySprite_walk_left		;; anim id 6 -> andando izquierda

			; ...

mySprite_stop_right	db	1				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0				;; frame id (multiplo de 4)

mySprite_stop_left	db	1				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	4				;; frame id (multiplo de 4)

mySprite_walk_right	db	8				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0,   8,  16,  8,  0, 24, 32, 24	;; frame id (multiplo de 4)

mySprite_walk_left	db	8				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	4,  12,  20, 12,  4, 28, 36, 28	;; frame id (multiplo de 4)

			; ...

			ALIGN 	4

			;; DESCRIPCION DE FRAMES DE ANIMACION

mySprite_frames		;;	frame par			frame impar
			dw	spr_mario_stop_par,		spr_mario_stop_impar		;; frame id 0
			dw	spr_mario_stop_i_par,		spr_mario_stop_i_impar		;; frame id 4

			dw	spr_mario_walk01_par,		spr_mario_walk01_impar		;; frame id 8
			dw	spr_mario_walk01_i_par,		spr_mario_walk01_i_impar	;; frame id 12

			dw	spr_mario_walk02_par,		spr_mario_walk02_impar		;; frame id 16
			dw	spr_mario_walk02_i_par,		spr_mario_walk02_i_impar	;; frame id 20

			dw	spr_mario_walk03_par,		spr_mario_walk03_impar		;; frame id 24
			dw	spr_mario_walk03_i_par,		spr_mario_walk03_i_impar	;; frame id 28

			dw	spr_mario_walk04_par,		spr_mario_walk04_impar		;; frame id 32
			dw	spr_mario_walk04_i_par,		spr_mario_walk04_i_impar	;; frame id 36
			; ...


