
	;; =========================================================================================
	;; ENEMIGO TORTUGA (colisiona con mapeado)
	;; =========================================================================================

	;; *  Los enemigos que colisionan con el mapeado tienen la limitacion de no poder moverse si
	;;    se encuentran fuera del area del mapeado cubierta por el buffer DECOMP_TILE_MAP, que
	;;    actualmente coincide con el area visible.

	;; *  Si se encuentra fuera del area visible pero dentro del margen definido en TURTLE_HMOV_AMP,
	;;    sigue activo pero sin moverse.

	;; *  Si se encuentra fuera del area visible y a mas distancia que TURTLE_HMOV_AMP, se
	;;    elimina.

	;; *  Su rectangulo para colisiones no debe ser muy ancho, ya que nada mas aparecer por los
	;;    margenes de la pantalla podria salirse del area visible y detectar colisones con
	;;    contenido residual de DECOMP_TILE_MAP. 

	;; CODIGO --------------------------------------------------------

	;; Metodos principales:

	;; * turtle_initSupport		-> Inicializacion de soporte para enemigos tipo tortugas
	;; * turtle_hi_create		-> Creacion de un enemigo tipo tortuga a la altura de una tuberia baja
	;; * turtle_low_create		-> Creacion de un enemigo tipo tortuga a nivel del suelo
	;; * turtle_process		-> Process de una tortuga


	;; Inicializacion de soporte para enemigos tipo tortugas

turtle_initSupport

	;; Inicializa un buffer de actors

	ld	ix,myTurtlesInfo		;; IX = puntero a lista de actors
	ld	e,TURTLE_INFO_LEN		;;  E = tamanyo de la lista
	ld	b,N_TURTLES_MAX			;;  B = numero de actors maximo
	ld	hl,turtle_fixed_info		;; HL = datos fijos
	jp	initActors

	;; Creacion de un enemigo tipo tortuga

	;; Entrada:
	;;	DE = puntero a guardar en enemy_gTileMap_adr_off
	;;	BC = coordenada X relativa donde debe poner el enemigo (0 / GAMEAREA_W_PIXEL)
	;; 	BC' = posicion en la pantalla de tiles (linea que va a dibujar)
	;; 	HL' = posicion en la pantalla de tiles (linea de la columna inversa con morralla)

	;; Variante que vuelta alto

turtle_hi_create
	ld	hl,120*#100
	jr	turtle_create

	;; Variante que vuela bajo

turtle_low_create
	ld	hl,144*#100

turtle_create
	ld	ix,myTurtlesInfo
	call	enemy_create

	ld	(ix + actor_state_off),ACTOR_STATE_WALKING
	ld	(ix + actor_gravity_state_off),ACTOR_GRAVITY_STATE_GOING_DOWN

	call	actor_init

	ld	(ix + actor_speed_off),-1

	ld	a,TURTLE_WALK_LEFT_ANIM_ID	;;  A = anim
	ld	b,0				;;  B = sec
	jp	setSpriteAnimSec


	;; Process de una tortuga

	;; Entrada:
	;; 	IX = estructura de datos de la tortuga

turtle_process
	bit	H_HIDDEN_BIT, (ix + actor_flags_off)
	jr	nz, turtle_h_hidden			

	call	turtle_move

	;; comprobacion de colisiones con el jugador

	call	checkPlayerActorCollision
	jp	c, enemy_remove
	
turtle_process_continue
	call	drawSprite
	jr	nc, turtle_no_h_hidden

	set	H_HIDDEN_BIT, (ix + actor_flags_off)
	jp	actor_process_back

turtle_no_h_hidden
	res	H_HIDDEN_BIT, (ix + actor_flags_off)
	jp	actor_process_back

	;; si no es visible en horizontal no lo mueve para no detectar colisiones fantasmas con tiles residuales de DECOMP_TILE_MAP

turtle_h_hidden
	call	actor_setCollisionHParams_red

	;; comprueba si se salio de la pantalla por los lados para eliminarlo

	ld	de,TURTLE_HMOV_AMP
	add	hl,de
	bit	7,h
	jp	nz, enemy_remove

	ld	de,-GAMEAREA_W_PIXEL - TURTLE_HMOV_AMP - TURTLE_HMOV_AMP
	add	hl,de
	jp	c, enemy_remove

	jr	turtle_process_continue

	;; Mueve la tortuga horizontalmente y avazan la animacion

turtle_move
	ld	a,(_counterFrame + 1)
	and	%011
	ld	a,0
	jr	nz, turtle_no_move_h
	ld	a,(ix + actor_speed_off)
turtle_no_move_h

	call	actor_process_h_motion
	;; Salida:
	;;	Flag C = 1 si hay colision
	;;		A = tipo de colision
	;;		HL = puntero al ID del tile en que colision
	jr	c, turtle_h_collision

	;; avanza la animacion

	ld	b,7
	call	sprite_process_anim

	jp	actor_process_gravity_motion

	;; colision horizontal

turtle_h_collision
	cp	RIGHT_COLLISION
	jr	nz, turtle_left_collision

	;; colision por la derecha

	ld	(ix + actor_speed_off),-1		;; cambia de direccion

	ld	a,TURTLE_WALK_LEFT_ANIM_ID
	jp	setSpriteAnim

turtle_left_collision

	;; colision por la izq

	ld	(ix + actor_speed_off),1		;; cambia de direccion

	ld	a,TURTLE_WALK_RIGHT_ANIM_ID
	jp	setSpriteAnim


	;; DATOS ---------------------------------------------------------

			; DATOS FIJOS

turtle_fixed_info	;; de sprite
			db	SPRITE_DRAW_MODE_AUTO		;;     = draw mode
			dw	turtle_frames			;; +2  = frames
			dw	turtle_anims			;; +4  = animaciones

			db	SPRITE_24W			;; +10 = ancho	(0 = 16, #10 = 20, ...)
			db	20				;; +11 = alto en pixeles
			db	24/8				;; +12 = ancho en tiles
			db	-5+1				;; +13 = pivotX
			db	-20				;; +14 = pivotY
			;; de actor
			db	-4+2				;; +34 = actor_collisionbb_xr_off
			db	-16				;; +35 = actor_collisionbb_yr_off
			db	8-3				;; +36 = actor_collisionbb_w_off
			db	15				;; +37 = actor_collisionbb_h_off

			;; ANIMACIONES

TURTLE_STOP_RIGHT_ANIM_ID	EQU	0
TURTLE_STOP_LEFT_ANIM_ID	EQU	2
TURTLE_WALK_RIGHT_ANIM_ID	EQU	4
TURTLE_WALK_LEFT_ANIM_ID	EQU	6

			ALIGN 	2

turtle_anims		dw	turtle_stop_right		;; anim id 0 -> parado derecha
			dw	turtle_stop_left		;; anim 1d 2 -> parado izquierda
			dw	turtle_walk_right		;; anim id 4 -> andando derecha
			dw	turtle_walk_left		;; anim id 6 -> andando izquierda
			; ...

turtle_stop_right	db	1				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0				;; frame id (multiplo de 4)

turtle_stop_left	db	1				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	4				;; frame id (multiplo de 4)

turtle_walk_right	db	5				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	0,   8,  16, 24, 32		;; frame id (multiplo de 4)

turtle_walk_left	db	5				;; numero de secuencias
			db	0				;; frame de inicio del loop
			db	4,  12,  20, 28, 36		;; frame id (multiplo de 4)

			; ...

			ALIGN 	4

			;; DESCRIPCION DE FRAMES DE ANIMACION

turtle_frames		;;	frame par			frame impar
			dw	spr_turtle_stop_par,		spr_turtle_stop_impar		;; frame id 0
			dw	spr_turtle_stop_i_par,		spr_turtle_stop_i_impar		;; frame id 4

			dw	spr_turtle_walk01_par,		spr_turtle_walk01_impar		;; frame id 8
			dw	spr_turtle_walk01_i_par,	spr_turtle_walk01_i_impar	;; frame id 12

			dw	spr_turtle_walk02_par,		spr_turtle_walk02_impar		;; frame id 16
			dw	spr_turtle_walk02_i_par,	spr_turtle_walk02_i_impar	;; frame id 20

			dw	spr_turtle_walk03_par,		spr_turtle_walk03_impar		;; frame id 24
			dw	spr_turtle_walk03_i_par,	spr_turtle_walk03_i_impar	;; frame id 28

			dw	spr_turtle_walk04_par,		spr_turtle_walk04_impar		;; frame id 32
			dw	spr_turtle_walk04_i_par,	spr_turtle_walk04_i_impar	;; frame id 36


